from qtpy import QtCore, QtWidgets

from .label import MLabel


class MForm(QtWidgets.QWidget):
    Horizontal = "horizontal"
    Vertical = "vertical"
    Inline = "inline"

    def __init__(
        self, layout: str | None = None, parent: QtWidgets.QWidget | None = None
    ):
        super().__init__(parent)
        layout = layout or MForm.Horizontal
        if layout == MForm.Inline:
            self._main_layout = QtWidgets.QHBoxLayout()
        elif layout == MForm.Vertical:
            self._main_layout = QtWidgets.QVBoxLayout()
        else:
            self._main_layout = QtWidgets.QFormLayout()
        self._model = None
        self._label_list: list[MLabel] = []

    def set_model(self, m: QtCore.QAbstractItemModel):
        self._model = m

    def set_label_align(self, align: QtCore.Qt.AlignmentFlag):
        for label in self._label_list:
            label.setAlignment(align)
        if isinstance(self._main_layout, QtWidgets.QFormLayout):
            self._main_layout.setLabelAlignment(align)

    @classmethod
    def horizontal(cls):
        return cls(layout=cls.Horizontal)

    @classmethod
    def vertical(cls):
        return cls(layout=cls.Vertical)

    @classmethod
    def inline(cls):
        return cls(layout=cls.Inline)



from qtpy import QtWidgets

from qtui.dayu.divider import MDivider
from qtui.dayu.field_mixin import MFieldMixin
from qtui.dayu.label import MLabel
from qtui.dayu.menu import MMenu
from qtui.dayu.push_button import MPushButton


class MenuExample(QtWidgets.QWidget, MFieldMixin):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self._init_ui()

    def _init_ui(self):
        self.register_field("button1_selected", "Beijing")
        self.register_field(
            "button1_selected_text", lambda: self.field("button1_selected")
        )
        button1 = MPushButton(text="Normal Menu")
        menu1 = MMenu(parent=self)
        menu1.set_data(["Beijing", "Shanghai", "Guangzhou", "Shenzhen"])
        button1.setMenu(menu1)
        button1.clicked.connect(button1.showMenu)
        label1 = MLabel()

        self.bind("button1_selected", menu1, "value", signal="value_changed")
        self.bind("button1_selected_text", label1, "text")

        self.register_field("button2_selected", ["Beijing"])
        self.register_field(
            "button2_selected_text", lambda: ", ".join(self.field("button2_selected"))
        )
        button2 = MPushButton(text="Multi Select Menu")
        menu2 = MMenu(exclusive=False, parent=self)
        menu2.set_data(["Beijing", "Shanghai", "Guangzhou", "Shenzhen"])
        button2.setMenu(menu2)
        button2.clicked.connect(button2.showMenu)
        label2 = MLabel()
        self.bind("button2_selected", menu2, "value", signal="value_changed")
        self.bind("button2_selected_text", label2, "text")

        self.register_field("button3_selected", "")
        self.register_field(
            "button3_selected_text", lambda: self.field("button3_selected")
        )
        button3 = MPushButton(text="Callback Function Get Options")
        menu3 = MMenu(parent=self)
        menu3.set_load_callback(
            lambda: ["Beijing", "Shanghai", "Guangzhou", "Shenzhen", "Seoul"]
        )
        button3.setMenu(menu3)
        button3.clicked.connect(button2.showMenu)
        label3 = MLabel()
        self.bind("button3_selected", menu3, "value", signal="value_changed")
        self.bind("button3_selected_text", label3, "text")

        self.register_field("button4_selected", "")
        self.register_field(
            "button4_selected_text", lambda: " / ".join(self.field("button4_selected"))
        )
        button4 = MPushButton(text="Cascader Select")
        menu4 = MMenu(cascader=True, parent=self)
        menu4.set_data(
            [
                "Beijing/Forbidden City",
                "Beijing/Tiananmen",
                "Beijing/Wangfujing",
                "Jiangsu/Nanjing/Confucius Temple",
                "Jiangsu/Suzhou/Zhuozhengyuan",
                "Jiangsu/Suzhou/Lion Grove",
            ]
        )
        button4.setMenu(menu4)
        button4.clicked.connect(button4.showMenu)
        label4 = MLabel()
        self.bind("button4_selected", menu4, "value", signal="value_changed")
        self.bind("button4_selected_text", label4, "text")

        sub_lay1 = QtWidgets.QHBoxLayout()
        sub_lay1.addWidget(button1)
        sub_lay1.addWidget(label1)
        sub_lay2 = QtWidgets.QHBoxLayout()
        sub_lay2.addWidget(button2)
        sub_lay2.addWidget(label2)
        sub_lay3 = QtWidgets.QHBoxLayout()
        sub_lay3.addWidget(button3)
        sub_lay3.addWidget(label3)
        sub_lay4 = QtWidgets.QHBoxLayout()
        sub_lay4.addWidget(button4)
        sub_lay4.addWidget(label4)

        sub_lay5 = QtWidgets.QHBoxLayout()
        button = MPushButton(text="Scroll Menu")
        menu = MMenu(parent=self)
        items = [
            "Beijing",
            "Shanghai",
            "Guangzhou",
            "Shenzhen",
            "Beidaihe",
            "BBC/data",
            "BBC/hello",
            "American",
        ]
        menu.set_data(items)
        menu.setProperty("max_scroll_count", 3)
        menu.setProperty("scrollable", True)
        menu.setProperty("searchable", True)
        button.setMenu(menu)
        sub_lay5.addWidget(button)

        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addWidget(MDivider("Select"))
        main_lay.addLayout(sub_lay1)
        main_lay.addLayout(sub_lay2)
        main_lay.addLayout(sub_lay3)
        main_lay.addWidget(MDivider("Cascader Select"))
        main_lay.addLayout(sub_lay4)
        main_lay.addLayout(sub_lay5)
        main_lay.addStretch()
        self.setLayout(main_lay)


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = MenuExample()
        MTheme().apply(test)
        test.show()



from typing import TYPE_CHECKING, Any, cast

from qtpy import QtCore, QtGui, QtWidgets

from . import utils
from .header_view import MHeaderView
from .item_model import MTableModel
from .menu import MMenu
from .qt import MPixmap, get_scale_factor
from .theme import MTheme
from .types import HeaderData, ItemViewMenuEvent

HEADER_SORT_MAP = {
    "asc": QtCore.Qt.SortOrder.AscendingOrder,
    "desc": QtCore.Qt.SortOrder.DescendingOrder,
}


class MAbstractView(QtWidgets.QAbstractItemView if TYPE_CHECKING else object):
    sig_context_menu = QtCore.Signal(object)

    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__()
        self.header_list: list[HeaderData] = []
        self.header_view: MHeaderView | None = None
        self._no_data_text = self.tr("No Data")
        self._no_data_image: QtGui.QPixmap | None = None

    def set_header_list(self, header_list: list[HeaderData]):
        scale_x, _ = get_scale_factor()
        self.header_list = header_list
        if self.header_view:
            for index, i in enumerate(header_list):
                self.header_view.setSectionHidden(index, i.get("hide", False))
                self.header_view.resizeSection(
                    index, int(i.get("width", 100) * scale_x)
                )
                if "order" in i:
                    order = i.get("order")
                    if order in HEADER_SORT_MAP.values():
                        self.header_view.setSortIndicator(
                            index, cast(QtCore.Qt.SortOrder, order)
                        )
                    elif order in HEADER_SORT_MAP:
                        self.header_view.setSortIndicator(index, HEADER_SORT_MAP[order])
                if i.get("selectable", False):
                    delegate = MOptionDelegate(parent=self)
                    delegate.set_exclusive(i.get("exclusive", True))
                    self.setItemDelegateForColumn(index, delegate)
                elif self.itemDelegateForColumn(index):
                    self.setItemDelegateForColumn(index, None)  # pyright: ignore[reportArgumentType]

    @QtCore.Slot(QtCore.QPoint)
    def slot_context_menu(self, point: QtCore.QPoint):
        proxy_index = self.indexAt(point)
        if proxy_index.isValid():
            selection: list[QtCore.QItemSelection] = []
            for index in (
                self.selectionModel().selectedRows()
                or self.selectionModel().selectedIndexes()
            ):
                data_obj = utils.real_index(index).internalPointer()
                selection.append(data_obj)
            event = ItemViewMenuEvent(view=self, selection=selection, extra={})
            self.sig_context_menu.emit(event)
        else:
            event = ItemViewMenuEvent(view=self, selection=[], extra={})
            self.sig_context_menu.emit(event)

    def enable_context_menu(self, enable: bool):
        if enable:
            self.setContextMenuPolicy(QtCore.Qt.ContextMenuPolicy.CustomContextMenu)
            self.customContextMenuRequested.connect(self.slot_context_menu)
        else:
            self.setContextMenuPolicy(QtCore.Qt.ContextMenuPolicy.NoContextMenu)

    def set_no_data_text(self, text: str):
        self._no_data_text = text

    def set_no_data_image(self, image: QtGui.QPixmap):
        self._no_data_image = image

    def paintEvent(self, event: QtGui.QPaintEvent, /):  # noqa: N802
        """
        Override paintEvent when there is no data to show,
        draw the preset picture and text.
        """
        model = utils.real_model(self.model())
        if model is None or (
            isinstance(model, MTableModel) and not model.get_data_list()
        ):
            draw_empty_content(self.viewport(), self._no_data_text, self._no_data_image)
        return super().paintEvent(event)


def draw_empty_content(
    view: QtWidgets.QWidget,
    text: str | None = None,
    pix_map: QtGui.QPixmap | None = None,
):
    pix_map = pix_map or MPixmap("empty.svg")
    text = text or view.tr("No Data")
    painter = QtGui.QPainter(view)
    font_metrics = painter.fontMetrics()
    painter.setPen(QtGui.QPen(QtGui.QColor(MTheme().secondary_text_color)))
    content_height = pix_map.height() + font_metrics.height()
    padding = 10
    proper_min_size = min(
        view.height() - padding * 2, view.width() - padding * 2, content_height
    )
    if proper_min_size < content_height:
        pix_map = pix_map.scaledToHeight(
            proper_min_size - font_metrics.height(),
            QtCore.Qt.TransformationMode.SmoothTransformation,
        )
        content_height = proper_min_size
    painter.drawText(
        int(view.width() / 2 - font_metrics.boundingRect(text).width() / 2),
        int(view.height() / 2 + content_height / 2 - font_metrics.height() / 2),
        text,
    )
    painter.drawPixmap(
        int(view.width() / 2 - pix_map.width() / 2),
        int(view.height() / 2 - content_height / 2),
        pix_map,
    )
    painter.end()


class MOptionDelegate(QtWidgets.QStyledItemDelegate):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.showed = False
        self.exclusive = True
        self.parent_widget: QtWidgets.QWidget | None = None
        self.arrow_space = 20
        self.arrow_height = 6

    def set_exclusive(self, flag: bool):
        self.exclusive = flag

    def createEditor(  # noqa: N802  # pyright: ignore[reportIncompatibleMethodOverride]
        self,
        parent: QtWidgets.QWidget,
        option: QtWidgets.QStyleOptionViewItem,
        index: QtCore.QModelIndex,
    ):
        self.parent_widget = parent
        editor = MMenu(exclusive=self.exclusive, parent=parent)
        editor.setWindowFlags(
            QtCore.Qt.WindowType.FramelessWindowHint | QtCore.Qt.WindowType.Window
        )
        model = cast(MAbstractView, utils.real_model(index))
        real_index = utils.real_index(index)
        data_obj = real_index.internalPointer()
        attr = f"{model.header_list[real_index.column()].get('key')}_list"

        editor.set_data(utils.get_obj_value(data_obj, attr, []))
        editor.value_changed.connect(
            self._slot_finish_edit,
        )
        return editor

    def setEditorData(  # noqa: N802  # pyright: ignore[reportIncompatibleMethodOverride]
        self, editor: MMenu, index: QtCore.QModelIndex
    ):
        editor.set_value(index.data(QtCore.Qt.ItemDataRole.EditRole))

    def setModelData(  # noqa: N802  # pyright: ignore[reportIncompatibleMethodOverride]
        self, editor: MMenu, model: QtCore.QAbstractItemModel, index: QtCore.QModelIndex
    ):
        model.setData(index, editor.property("value"))

    def updateEditorGeometry(  # noqa: N802  # pyright: ignore[reportIncompatibleMethodOverride]
        self,
        editor: MMenu,
        option: QtWidgets.QStyleOptionViewItem,
        index: QtCore.QModelIndex,
    ):
        if self.parent_widget is None:
            return
        editor.move(
            self.parent_widget.mapToGlobal(
                QtCore.QPoint(option.rect.x(), option.rect.y() + option.rect.height())  # pyright: ignore[reportUnknownMemberType, reportUnknownArgumentType, reportAttributeAccessIssue]
            )
        )

    def paint(  # pyright: ignore[reportIncompatibleMethodOverride]
        self,
        painter: QtGui.QPainter,
        option: QtWidgets.QStyleOptionViewItem,
        index: QtCore.QModelIndex,
    ):
        painter.save()
        icon_color = MTheme().icon_color
        if option.state & QtWidgets.QStyle.StateFlag.State_MouseOver:  # pyright: ignore[reportUnknownMemberType, reportAttributeAccessIssue]
            painter.fillRect(option.rect, QtGui.QColor(MTheme().primary_5))  # pyright: ignore[reportUnknownMemberType, reportUnknownArgumentType, reportAttributeAccessIssue]
            icon_color = "#fff"
        if option.state & QtWidgets.QStyle.StateFlag.State_Selected:  # pyright: ignore[reportUnknownMemberType, reportAttributeAccessIssue]
            painter.fillRect(option.rect, QtGui.QColor(MTheme().primary_6))  # pyright: ignore[reportUnknownMemberType, reportUnknownArgumentType, reportAttributeAccessIssue]
            icon_color = "#fff"
        painter.setRenderHint(QtGui.QPainter.RenderHint.Antialiasing)
        painter.setPen(QtCore.Qt.PenStyle.NoPen)
        painter.setBrush(QtGui.QBrush(QtCore.Qt.GlobalColor.white))
        pix = MPixmap("down_fill.svg", icon_color)
        h = cast(int, option.rect.height())  # pyright: ignore[reportUnknownMemberType, reportAttributeAccessIssue]
        pix = pix.scaledToWidth(
            int(h * 0.5), QtCore.Qt.TransformationMode.SmoothTransformation
        )
        painter.drawPixmap(
            int(option.rect.x() + option.rect.width() - h),  # pyright: ignore[reportUnknownArgumentType, reportUnknownMemberType, reportAttributeAccessIssue]
            int(option.rect.y() + h / 4),  # pyright: ignore[reportUnknownArgumentType, reportUnknownMemberType, reportAttributeAccessIssue]
            pix,
        )
        painter.restore()
        super().paint(painter, option, index)

    @QtCore.Slot(object)
    def _slot_finish_edit(self, obj: Any):
        self.commitData.emit(self.sender())

    def sizeHint(  # noqa: N802  # pyright: ignore[reportIncompatibleMethodOverride]
        self, option: QtWidgets.QStyleOptionViewItem, index: QtCore.QModelIndex
    ):
        orig = super().sizeHint(option, index)
        return QtCore.QSize(orig.width() + self.arrow_space, orig.height())


class MTableView(QtWidgets.QTableView, MAbstractView):
    def __init__(
        self,
        size: int | None = None,
        show_row_count: bool = False,
        parent: QtWidgets.QWidget | None = None,
    ):
        super().__init__(parent)
        size = size or MTheme().default_size
        ver_header_view = MHeaderView(QtCore.Qt.Orientation.Vertical, parent=self)
        ver_header_view.setDefaultSectionSize(size)
        ver_header_view.setSortIndicatorShown(False)
        self.setVerticalHeader(ver_header_view)
        self.header_view = MHeaderView(QtCore.Qt.Orientation.Horizontal, parent=self)
        self.header_view.setFixedHeight(size)
        if not show_row_count:
            ver_header_view.hide()
        self.setHorizontalHeader(self.header_view)
        self.setSortingEnabled(True)
        self.setSelectionBehavior(
            QtWidgets.QAbstractItemView.SelectionBehavior.SelectRows
        )
        self.setAlternatingRowColors(True)
        self.setShowGrid(False)

    def setShowGrid(self, flag: bool):  # noqa: N802
        if self.header_view is None:
            return
        self.header_view.setProperty("grid", flag)
        self.verticalHeader().setProperty("grid", flag)
        self.header_view.style().polish(self.header_view)

        return super().setShowGrid(flag)


class MTreeView(QtWidgets.QTreeView, MAbstractView):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self._no_data_image: QtGui.QPixmap | None = None
        self._no_data_text = self.tr("No Data")
        self.header_view = MHeaderView(QtCore.Qt.Orientation.Horizontal)
        self.setHeader(self.header_view)
        self.setSortingEnabled(True)
        self.setAlternatingRowColors(True)

    def set_no_data_text(self, text: str):
        self._no_data_text = text


class MBigView(QtWidgets.QListView, MAbstractView):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setViewMode(QtWidgets.QListView.ViewMode.IconMode)
        self.setResizeMode(QtWidgets.QListView.ResizeMode.Adjust)
        self.setMovement(QtWidgets.QListView.Movement.Static)
        self.setSpacing(10)
        default_size = MTheme().big_view_default_size
        self.setIconSize(QtCore.QSize(default_size, default_size))

    def scale_size(self, factor: float):
        """Scale the icon size."""
        new_size = self.iconSize() * factor
        max_size = MTheme().big_view_max_size
        min_size = MTheme().big_view_min_size
        if new_size.width() > max_size:
            new_size = QtCore.QSize(max_size, max_size)
        elif new_size.width() < min_size:
            new_size = QtCore.QSize(min_size, min_size)
        self.setIconSize(new_size)

    def wheelEvent(self, event: QtGui.QWheelEvent):  # noqa: N802
        """Override wheelEvent while user press ctrl, zoom the list view icon size."""
        if event.modifiers() == QtCore.Qt.KeyboardModifier.ControlModifier:
            num_degrees = event.angleDelta().y() / 8.0
            num_steps = num_degrees / 15.0
            factor = pow(1.125, num_steps)
            self.scale_size(factor)
        else:
            super().wheelEvent(event)


class MListView(QtWidgets.QListView, MAbstractView):
    def __init__(
        self, size: int | None = None, parent: QtWidgets.QWidget | None = None
    ):
        super().__init__(parent)
        self.setProperty("dayu_size", size or MTheme().default_size)
        self.setModelColumn(0)
        self.setAlternatingRowColors(True)

    def set_show_column(self, attr: str):
        for index, attr_dict in enumerate(self.header_list):
            if attr_dict.get("key") == attr:
                self.setModelColumn(index)
                break
        else:
            self.setModelColumn(0)



"""MDockWidget"""

from qtpy import QtCore, QtWidgets


class MDockWidget(QtWidgets.QDockWidget):
    """
    Just apply the qss. No more extend.
    """

    def __init__(
        self,
        title: str = "",
        parent: QtWidgets.QWidget | None = None,
        flags: QtCore.Qt.WindowType = QtCore.Qt.WindowType.Widget,
    ):
        super().__init__(title, parent=parent, flags=flags)



from qtpy import QtCore, QtWidgets

from qtui.dayu.button_group import MRadioButtonGroup
from qtui.dayu.check_box import MCheckBox
from qtui.dayu.combo_box import MComboBox
from qtui.dayu.divider import MDivider
from qtui.dayu.field_mixin import MFieldMixin
from qtui.dayu.line_edit import MLineEdit
from qtui.dayu.push_button import MPushButton
from qtui.dayu.qt import MIcon
from qtui.dayu.slider import MSlider
from qtui.dayu.spin_box import MDateEdit, MSpinBox
from qtui.dayu.switch import MSwitch
from qtui.dayu.theme import MTheme
from qtui.dayu.types import MButtonGroupData


class ThemeExample(QtWidgets.QWidget, MFieldMixin):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self._init_ui()

    def _init_ui(self):
        form_lay = QtWidgets.QFormLayout()
        form_lay.setLabelAlignment(QtCore.Qt.AlignmentFlag.AlignRight)
        gender_grp = MRadioButtonGroup()
        gender_grp.set_button_list(
            [
                MButtonGroupData(text="Female", icon=MIcon("female.svg")),
                MButtonGroupData(text="Male", icon=MIcon("male.svg")),
            ]
        )

        country_combo_box = MComboBox().small()
        country_combo_box.addItems(["China", "France", "Japan", "US"])
        date_edit = MDateEdit().small()
        date_edit.setCalendarPopup(True)

        form_lay.addRow("Name:", MLineEdit().small())
        form_lay.addRow("Gender:", gender_grp)
        form_lay.addRow("Age:", MSpinBox().small())
        form_lay.addRow("Password:", MLineEdit().small().password())
        form_lay.addRow("Country:", country_combo_box)
        form_lay.addRow("Birthday:", date_edit)
        switch = MSwitch()
        switch.setChecked(True)
        form_lay.addRow("Switch:", switch)
        slider = MSlider()
        slider.setValue(30)
        form_lay.addRow("Slider:", slider)

        button_change = MPushButton(text="Change Theme")
        button_change.clicked.connect(self.slot_change_theme)

        button_lay = QtWidgets.QHBoxLayout()
        button_lay.addStretch()
        button_lay.addWidget(MPushButton(text="Submit").primary())
        button_lay.addWidget(MPushButton(text="Cancel"))

        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addLayout(form_lay)
        main_lay.addWidget(MCheckBox("I accept the terms and conditions"))
        main_lay.addWidget(button_change)
        main_lay.addStretch()
        main_lay.addWidget(MDivider())
        main_lay.addLayout(button_lay)
        self.setLayout(main_lay)

    def slot_change_theme(self):
        import random

        color = random.choice(list(MTheme.GlobalColor))
        theme = random.choice(list(MTheme.ThemeType))
        MTheme().change_theme(self.window(), theme, color)


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = ThemeExample()
        MTheme().apply(test)
        test.show()



"""
Custom Stylesheet for QSpinBox, QDoubleSpinBox, QDateTimeEdit, QDateEdit, QTimeEdit.
Only add size arg for their __init__.
"""

from qtpy import QtCore, QtWidgets

from .mixin import cursor_mixin
from .theme import MTheme


@cursor_mixin
class MSpinBox(QtWidgets.QSpinBox):
    """
    MSpinBox just use stylesheet and add dayu_size. No more extend.
    Property:
        dayu_size: The height of MSpinBox
    """

    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent=parent)
        self._dayu_size: int = MTheme().default_size

    def get_dayu_size(self) -> int:
        """
        Get the MSpinBox height
        :return: integer
        """
        return self._dayu_size

    def set_dayu_size(self, value: int) -> None:
        """
        Set the MSpinBox size.
        :param value: integer
        :return: None
        """
        self._dayu_size = value
        self.style().polish(self)

    dayu_size = QtCore.Property(int, get_dayu_size, set_dayu_size)

    def huge(self):
        """Set MSpinBox to huge size"""
        self.set_dayu_size(MTheme().huge)
        return self

    def large(self):
        """Set MSpinBox to large size"""
        self.set_dayu_size(MTheme().large)
        return self

    def medium(self):
        """Set MSpinBox to  medium"""
        self.set_dayu_size(MTheme().medium)
        return self

    def small(self):
        """Set MSpinBox to small size"""
        self.set_dayu_size(MTheme().small)
        return self

    def tiny(self):
        """Set MSpinBox to tiny size"""
        self.set_dayu_size(MTheme().tiny)
        return self


@cursor_mixin
class MDoubleSpinBox(QtWidgets.QDoubleSpinBox):
    """
    MDoubleSpinBox just use stylesheet and add dayu_size. No more extend.
    Property:
        dayu_size: The height of MDoubleSpinBox
    """

    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent=parent)
        self._dayu_size: int = MTheme().default_size

    def get_dayu_size(self) -> int:
        """
        Get the MDoubleSpinBox height
        :return: integer
        """
        return self._dayu_size

    def set_dayu_size(self, value: int) -> None:
        """
        Set the MDoubleSpinBox size.
        :param value: integer
        :return: None
        """
        self._dayu_size = value
        self.style().polish(self)

    dayu_size = QtCore.Property(int, get_dayu_size, set_dayu_size)

    def huge(self):
        """Set MDoubleSpinBox to huge size"""
        self.set_dayu_size(MTheme().huge)
        return self

    def large(self):
        """Set MDoubleSpinBox to large size"""
        self.set_dayu_size(MTheme().large)
        return self

    def medium(self):
        """Set MDoubleSpinBox to  medium"""
        self.set_dayu_size(MTheme().medium)
        return self

    def small(self):
        """Set MDoubleSpinBox to small size"""
        self.set_dayu_size(MTheme().small)
        return self

    def tiny(self):
        """Set MDoubleSpinBox to tiny size"""
        self.set_dayu_size(MTheme().tiny)
        return self


@cursor_mixin
class MDateTimeEdit(QtWidgets.QDateTimeEdit):
    """
    MDateTimeEdit just use stylesheet and add dayu_size. No more extend.
    Property:
        dayu_size: The height of MDateTimeEdit
    """

    def __init__(
        self,
        datetime: QtCore.QDateTime | None = None,
        parent: QtWidgets.QWidget | None = None,
    ):
        if datetime is None:
            super().__init__(parent=parent)
        else:
            super().__init__(datetime, parent=parent)
        self._dayu_size: int = MTheme().default_size

    def get_dayu_size(self) -> int:
        """
        Get the MDateTimeEdit height
        :return: integer
        """
        return self._dayu_size

    def set_dayu_size(self, value: int) -> None:
        """
        Set the MDateTimeEdit size.
        :param value: integer
        :return: None
        """
        self._dayu_size = value
        self.style().polish(self)

    dayu_size = QtCore.Property(int, get_dayu_size, set_dayu_size)

    def huge(self):
        """Set MDateTimeEdit to huge size"""
        self.set_dayu_size(MTheme().huge)
        return self

    def large(self):
        """Set MDateTimeEdit to large size"""
        self.set_dayu_size(MTheme().large)
        return self

    def medium(self):
        """Set MDateTimeEdit to  medium"""
        self.set_dayu_size(MTheme().medium)
        return self

    def small(self):
        """Set MDateTimeEdit to small size"""
        self.set_dayu_size(MTheme().small)
        return self

    def tiny(self):
        """Set MDateTimeEdit to tiny size"""
        self.set_dayu_size(MTheme().tiny)
        return self


@cursor_mixin
class MDateEdit(QtWidgets.QDateEdit):
    """
    MDateEdit just use stylesheet and add dayu_size. No more extend.
    Property:
        dayu_size: The height of MDateEdit
    """

    def __init__(
        self,
        date: QtCore.QDate | None = None,
        parent: QtWidgets.QWidget | None = None,
    ):
        if date is None:
            super().__init__(parent=parent)
        else:
            super().__init__(date, parent=parent)
        self._dayu_size: int = MTheme().default_size

    def get_dayu_size(self) -> int:
        """
        Get the MDateEdit height
        :return: integer
        """
        return self._dayu_size

    def set_dayu_size(self, value: int) -> None:
        """
        Set the MDateEdit size.
        :param value: integer
        :return: None
        """
        self._dayu_size = value
        self.style().polish(self)

    dayu_size = QtCore.Property(int, get_dayu_size, set_dayu_size)

    def huge(self):
        """Set MDateEdit to huge size"""
        self.set_dayu_size(MTheme().huge)
        return self

    def large(self):
        """Set MDateEdit to large size"""
        self.set_dayu_size(MTheme().large)
        return self

    def medium(self):
        """Set MDateEdit to  medium"""
        self.set_dayu_size(MTheme().medium)
        return self

    def small(self):
        """Set MDateEdit to small size"""
        self.set_dayu_size(MTheme().small)
        return self

    def tiny(self):
        """Set MDateEdit to tiny size"""
        self.set_dayu_size(MTheme().tiny)
        return self


@cursor_mixin
class MTimeEdit(QtWidgets.QTimeEdit):
    """
    MTimeEdit just use stylesheet and add dayu_size. No more extend.
    Property:
        dayu_size: The height of MTimeEdit
    """

    def __init__(
        self,
        time: QtCore.QTime | None = None,
        parent: QtWidgets.QWidget | None = None,
    ):
        if time is None:
            super().__init__(parent=parent)
        else:
            super().__init__(time, parent=parent)
        self._dayu_size: int = MTheme().default_size

    def get_dayu_size(self) -> int:
        """
        Get the MTimeEdit height
        :return: integer
        """
        return self._dayu_size

    def set_dayu_size(self, value: int) -> None:
        """
        Set the MTimeEdit size.
        :param value: integer
        :return: None
        """
        self._dayu_size = value
        self.style().polish(self)

    dayu_size = QtCore.Property(int, get_dayu_size, set_dayu_size)

    def huge(self):
        """Set MTimeEdit to huge size"""
        self.set_dayu_size(MTheme().huge)
        return self

    def large(self):
        """Set MTimeEdit to large size"""
        self.set_dayu_size(MTheme().large)
        return self

    def medium(self):
        """Set MTimeEdit to  medium"""
        self.set_dayu_size(MTheme().medium)
        return self

    def small(self):
        """Set MTimeEdit to small size"""
        self.set_dayu_size(MTheme().small)
        return self

    def tiny(self):
        """Set MTimeEdit to tiny size"""
        self.set_dayu_size(MTheme().tiny)
        return self



import functools
from collections.abc import Sequence
from typing import Generic, TypeVar, cast

from qtpy import QtCore, QtGui, QtWidgets

from .check_box import MCheckBox
from .menu import MMenu
from .push_button import MPushButton
from .qt import get_scale_factor
from .radio_button import MRadioButton
from .tool_button import MToolButton
from .types import MButtonGroupData
from .widget import MWidget

T = TypeVar("T", bound=QtWidgets.QAbstractButton)


class MButtonGroup(QtWidgets.QButtonGroup, Generic[T]):
    def __init__(
        self,
        parent: QtWidgets.QWidget | None = None,
    ):
        super().__init__(parent=parent)

    def button(self, id: int) -> T:
        return cast(T, super().button(id))

    def buttons(self) -> list[T]:  # pyright: ignore[reportIncompatibleMethodOverride]
        return [cast(T, b) for b in super().buttons()]


class MButtonGroupBase(QtWidgets.QWidget, MWidget, Generic[T]):  # pyright: ignore[reportGeneralTypeIssues]
    def __init__(
        self,
        orientation: QtCore.Qt.Orientation = QtCore.Qt.Orientation.Horizontal,
        parent: QtWidgets.QWidget | None = None,
    ):
        super().__init__(parent=parent)

        self._main_layout = QtWidgets.QBoxLayout(
            QtWidgets.QBoxLayout.Direction.LeftToRight
            if orientation == QtCore.Qt.Orientation.Horizontal
            else QtWidgets.QBoxLayout.Direction.TopToBottom
        )

        self._main_layout.setContentsMargins(0, 0, 0, 0)

        self.setLayout(self._main_layout)

        self.setSizePolicy(
            QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Minimum
        )

        self._button_group = MButtonGroup[T]()

        self._orientation = (
            "horizontal"
            if orientation == QtCore.Qt.Orientation.Horizontal
            else "vertical"
        )

    def set_spacing(self, value: int) -> None:
        self._main_layout.setSpacing(value)

    def get_button_group(self) -> QtWidgets.QButtonGroup:
        return self._button_group

    def get_button(self, id: int) -> T:
        return self._button_group.button(id)

    def get_buttons(self) -> list[T]:
        return self._button_group.buttons()

    def create_button(self, data_dict: MButtonGroupData) -> T:
        raise NotImplementedError()

    def add_button(
        self, data_dict: MButtonGroupData | str | QtGui.QIcon, index: int | None = None
    ) -> T:
        if isinstance(data_dict, str):
            data_dict = MButtonGroupData(text=data_dict)

        elif isinstance(data_dict, QtGui.QIcon):
            data_dict = MButtonGroupData(icon=data_dict)

        button = self.create_button(data_dict)
        button.setProperty("combine", self._orientation)

        text = data_dict.get("text")
        icon = data_dict.get("icon")
        data = data_dict.get("data")
        checked = data_dict.get("checked")
        shortcut = data_dict.get("shortcut")
        tooltip = data_dict.get("tooltip")
        checkable = data_dict.get("checkable")
        clicked = data_dict.get("clicked")
        toggled = data_dict.get("toggled")

        if text:
            button.setProperty("text", text)

        if icon:
            button.setProperty("icon", icon)

        if data:
            button.setProperty("data", data)

        if checked:
            button.setProperty("checked", checked)

        if shortcut:
            button.setProperty("shortcut", shortcut)

        if tooltip:
            button.setProperty("toolTip", tooltip)

        if checkable:
            button.setProperty("checkable", checkable)

        if clicked:
            button.clicked.connect(clicked)

        if toggled:
            button.toggled.connect(toggled)

        if index is None:
            self._button_group.addButton(button)
        else:
            self._button_group.addButton(button, index)

        self._main_layout.insertWidget(self._main_layout.count(), button)

        return button

    def set_button_list(
        self, button_list: Sequence[MButtonGroupData | str | QtGui.QIcon]
    ) -> None:
        for button in self.get_buttons():
            self._button_group.removeButton(button)
            self._main_layout.removeWidget(button)
            button.setVisible(False)

        for index, data_dict in enumerate(button_list):
            button = self.add_button(data_dict, index)

            if index == 0:
                button.setProperty("position", "left")
            elif index == len(button_list) - 1:
                button.setProperty("position", "right")
            else:
                button.setProperty("position", "center")


class MPushButtonGroup(MButtonGroupBase[MPushButton]):
    def __init__(
        self,
        orientation: QtCore.Qt.Orientation = QtCore.Qt.Orientation.Horizontal,
        parent: QtWidgets.QWidget | None = None,
    ):
        super().__init__(orientation=orientation, parent=parent)

        self.set_spacing(1)
        self.set_dayu_type(MPushButton.ButtonType.Primary)
        self._button_group.setExclusive(False)

    def create_button(self, data_dict: MButtonGroupData) -> MPushButton:
        button = MPushButton()
        button.set_dayu_size(data_dict.get("dayu_size", self._dayu_size))
        button.set_dayu_type(data_dict.get("dayu_type", self._dayu_type))

        return button


class MCheckBoxGroup(MButtonGroupBase[MCheckBox]):
    checked_changed = QtCore.Signal(list)

    def __init__(
        self,
        orientation: QtCore.Qt.Orientation = QtCore.Qt.Orientation.Horizontal,
        parent: QtWidgets.QWidget | None = None,
    ):
        super().__init__(orientation=orientation, parent=parent)
        scale_x, _ = get_scale_factor()
        self.set_spacing(int(15 * scale_x))
        self._button_group.setExclusive(False)
        self.setContextMenuPolicy(QtCore.Qt.ContextMenuPolicy.CustomContextMenu)

        self._dayu_checked = []
        self._context_menu_show = True
        self._context_menu_select_all = "Select All"
        self._context_menu_select_none = "Select None"
        self._context_menu_select_invert = "Select Invert"

        self.customContextMenuRequested.connect(self._on_context_menu_requested)
        self._button_group.buttonClicked.connect(self._on_map_signal)

    def create_button(self, data_dict: MButtonGroupData) -> MCheckBox:
        return MCheckBox()

    def set_context_menu_properties(
        self,
        show: bool = True,
        select_all: str = "Select All",
        select_none: str = "Select None",
        select_invert: str = "Select Invert",
    ) -> None:
        self._context_menu_show = show
        self._context_menu_select_all = select_all
        self._context_menu_select_none = select_none
        self._context_menu_select_invert = select_invert
        if show:
            self.setContextMenuPolicy(QtCore.Qt.ContextMenuPolicy.CustomContextMenu)
        else:
            self.setContextMenuPolicy(QtCore.Qt.ContextMenuPolicy.NoContextMenu)

    @QtCore.Slot(QtCore.QPoint)
    def _on_context_menu_requested(self, point: QtCore.QPoint) -> None:
        context_menu = MMenu(parent=self)
        action_select_all = context_menu.addAction(self._context_menu_select_all)
        action_select_none = context_menu.addAction(self._context_menu_select_none)
        action_select_invert = context_menu.addAction(self._context_menu_select_invert)
        action_select_all.triggered.connect(
            functools.partial(self._on_set_select, True)
        )
        action_select_none.triggered.connect(
            functools.partial(self._on_set_select, False)
        )
        action_select_invert.triggered.connect(
            functools.partial(self._on_set_select, None)
        )
        context_menu.exec(QtGui.QCursor.pos() + QtCore.QPoint(10, 10))

    @QtCore.Slot(bool)
    def _on_set_select(self, state: bool | None) -> None:
        for check_box in self.get_buttons():
            if state is None:
                old_state = check_box.isChecked()
                check_box.setChecked(not old_state)
            else:
                check_box.setChecked(state)
        self._on_map_signal()

    @QtCore.Slot(int)
    def _on_map_signal(self, state: int | None = None) -> None:
        self.checked_changed.emit(
            [
                check_box.text()
                for check_box in self.get_buttons()
                if check_box.isChecked()
            ]
        )

    def set_dayu_checked(self, value: list[str] | str) -> None:
        if not isinstance(value, list):
            value = [value]

        if value == self.get_dayu_checked():
            return

        self._dayu_checked = value
        for check_box in self.get_buttons():
            flag = (
                QtCore.Qt.CheckState.Checked
                if check_box.text() in value
                else QtCore.Qt.CheckState.Unchecked
            )

            if flag != check_box.checkState():
                check_box.setCheckState(flag)

        self.checked_changed.emit(value)

    def get_dayu_checked(self) -> list[str]:
        return [
            check_box.text()
            for check_box in self.get_buttons()
            if check_box.isChecked()
        ]

    dayu_checked = QtCore.Property(
        "QVariantList",  # pyright: ignore[reportArgumentType]
        get_dayu_checked,
        set_dayu_checked,
        notify=checked_changed,
    )


class MRadioButtonGroup(MButtonGroupBase[MRadioButton]):
    """
    Property:
        dayu_checked
    """

    checked_changed = QtCore.Signal(int)

    def __init__(
        self,
        orientation: QtCore.Qt.Orientation = QtCore.Qt.Orientation.Horizontal,
        parent: QtWidgets.QWidget | None = None,
    ):
        super().__init__(orientation=orientation, parent=parent)
        scale_x, _ = get_scale_factor()
        self.set_spacing(int(15 * scale_x))
        self._button_group.setExclusive(True)
        self._button_group.buttonClicked.connect(
            lambda: self.checked_changed.emit(self._button_group.checkedId())
        )

    def create_button(self, data_dict: MButtonGroupData) -> MRadioButton:
        return MRadioButton()

    def set_dayu_checked(self, value: int) -> None:
        if value == self.get_dayu_checked():
            return

        button = self._button_group.button(value)
        if button:
            button.setChecked(True)
            self.checked_changed.emit(value)
        else:
            print("error")

    def get_dayu_checked(self) -> int:
        return self._button_group.checkedId()

    dayu_checked = QtCore.Property(
        int,
        get_dayu_checked,
        set_dayu_checked,
        notify=checked_changed,
    )


class MToolButtonGroup(MButtonGroupBase[MToolButton]):
    checked_changed = QtCore.Signal(int)

    def __init__(
        self,
        size: int | None = None,
        type: str | None = None,
        exclusive: bool = False,
        orientation: QtCore.Qt.Orientation = QtCore.Qt.Orientation.Horizontal,
        parent: QtWidgets.QWidget | None = None,
    ):
        super().__init__(orientation=orientation, parent=parent)
        self.set_spacing(1)
        self._button_group.setExclusive(exclusive)
        self._size = size
        self._type = type
        self._button_group.buttonClicked.connect(
            lambda: self.checked_changed.emit(self._button_group.checkedId())
        )

    def create_button(self, data_dict: MButtonGroupData) -> MToolButton:
        button = MToolButton()
        svg = data_dict.get("svg")
        text = data_dict.get("text")
        icon = data_dict.get("icon")
        if svg:
            button.svg(svg)
        if text:
            if svg or icon:
                button.text_beside_icon()
            else:
                button.text_only()
        else:
            button.icon_only()
        return button

    def set_dayu_checked(self, value: int) -> None:
        if value == self.get_dayu_checked():
            return
        button = self._button_group.button(value)
        if button:
            button.setChecked(True)
            self.checked_changed.emit(value)
        else:
            raise ValueError("button not found")

    def get_dayu_checked(self) -> int:
        return self._button_group.checkedId()

    dayu_checked = QtCore.Property(
        int,
        get_dayu_checked,
        set_dayu_checked,
        notify=checked_changed,
    )



import random

from qtpy import QtWidgets

from qtui.dayu.combo_box import MComboBox
from qtui.dayu.divider import MDivider
from qtui.dayu.field_mixin import MFieldMixin
from qtui.dayu.label import MLabel
from qtui.dayu.menu import MMenu
from qtui.dayu.theme import MTheme
from qtui.dayu.types import MenuItemData


class ComboBoxExample(QtWidgets.QWidget, MFieldMixin):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self._init_ui()

    def _init_ui(self):
        cities = ["Beijing", "Shanghai", "Guangzhou", "Shenzhen"]
        self.register_field("button1_selected", "Beijing")
        menu1 = MMenu(parent=self)
        menu1.set_data(cities)
        size_list = [
            ("Large", MTheme().large),
            ("Medium", MTheme().medium),
            ("Small", MTheme().small),
        ]
        size_lay = QtWidgets.QHBoxLayout()
        for _label, size in size_list:
            combo_box = MComboBox()
            combo_box.set_dayu_size(size)
            combo_box.set_menu(menu1)
            size_lay.addWidget(combo_box)
            self.bind("button1_selected", combo_box, "value", signal="value_changed")

        self.register_field("button2_selected", ["Beijing"])
        menu2 = MMenu(exclusive=False, parent=self)
        menu2.set_data(cities)
        select2 = MComboBox()
        select2.set_menu(menu2)
        self.bind("button2_selected", select2, "value", signal="value_changed")

        def dynamic_get_city() -> list[str]:
            data = [*cities, "Zhengzhou", "Shijiazhuang"]
            start = random.randint(0, len(data))
            end = random.randint(start, len(data))
            return data[start:end]

        self.register_field("button3_selected", "")
        menu3 = MMenu(parent=self)
        menu3.set_load_callback(dynamic_get_city)
        select3 = MComboBox()
        select3.set_menu(menu3)
        self.bind("button3_selected", select3, "value", signal="value_changed")

        menu4_items = [
            MenuItemData(
                children=[
                    MenuItemData(value="Forbidden City", label="Forbidden City"),
                    MenuItemData(value="Temple of Heaven", label="Temple of Heaven"),
                    MenuItemData(value="Wangfujing", label="Wangfujing"),
                ],
                value="Beijing",
                label="Beijing",
            ),
            MenuItemData(
                children=[
                    MenuItemData(
                        children=[
                            MenuItemData(
                                value="Confucius Temple",
                                label="Confucius Temple",
                            ),
                        ],
                        value="Nanjing",
                        label="Nanjing",
                    ),
                    MenuItemData(
                        children=[
                            MenuItemData(
                                value="Humble Garden",
                                label="Humble Garden",
                            ),
                            MenuItemData(
                                value="Lion Grove",
                                label="Lion Grove",
                            ),
                        ],
                        value="Suzhou",
                        label="Suzhou",
                    ),
                ],
                value="Jiangsu",
                label="Jiangsu",
            ),
        ]

        self.register_field("button4_selected", "")
        menu4 = MMenu(cascader=True, parent=self)
        menu4.set_data(menu4_items)
        select4 = MComboBox()
        select4.set_menu(menu4)
        select4.set_formatter(lambda x: " / ".join(x))
        self.bind("button4_selected", select4, "value", signal="value_changed")

        self.register_field("button5_selected", "")
        menu5 = MMenu(exclusive=False, parent=self)
        menu5.set_data(cities)
        select5 = MComboBox()
        select5.set_menu(menu5)
        select5.set_formatter(lambda x: " & ".join(x))
        self.bind("button5_selected", select5, "value", signal="value_changed")

        sub_lay1 = QtWidgets.QHBoxLayout()
        sub_lay1.addWidget(MLabel("Single Selection - Various Sizes"))
        sub_lay1.addLayout(size_lay)
        sub_lay1.addStretch()
        sub_lay2 = QtWidgets.QHBoxLayout()
        sub_lay2.addWidget(MLabel("Multiple Selection"))
        sub_lay2.addWidget(select2)
        sub_lay2.addStretch()
        sub_lay3 = QtWidgets.QHBoxLayout()
        sub_lay3.addWidget(MLabel("Dynamic Options"))
        sub_lay3.addWidget(select3)
        sub_lay3.addStretch()
        sub_lay4 = QtWidgets.QHBoxLayout()
        sub_lay4.addWidget(MLabel("Cascading Selection"))
        sub_lay4.addWidget(select4)
        sub_lay4.addStretch()
        sub_lay5 = QtWidgets.QHBoxLayout()
        sub_lay5.addWidget(MLabel("Custom Display"))
        sub_lay5.addWidget(select5)
        sub_lay5.addStretch()

        sub_lay6 = QtWidgets.QHBoxLayout()
        combo = MComboBox()
        items = [*cities, "Beidaihe"]

        items += ["a" * i for i in range(20)]
        combo.addItems(items)
        combo.setProperty("searchable", True)
        sub_lay6.addWidget(MLabel("Search Autocomplete"))
        sub_lay6.addWidget(combo)

        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addWidget(MDivider("Select"))
        main_lay.addLayout(sub_lay1)
        main_lay.addLayout(sub_lay2)
        main_lay.addLayout(sub_lay3)
        main_lay.addWidget(MDivider("Custom Format"))
        main_lay.addLayout(sub_lay4)
        main_lay.addLayout(sub_lay5)
        main_lay.addLayout(sub_lay6)
        main_lay.addStretch()

        self.setLayout(main_lay)


if __name__ == "__main__":
    from qtui.dayu.qt import application

    with application() as app:
        test = ComboBoxExample()
        MTheme().apply(test)
        test.show()



from qtpy import QtWidgets

from qtui.dayu.divider import MDivider
from qtui.dayu.field_mixin import MFieldMixin
from qtui.dayu.label import MLabel
from qtui.dayu.loading import MLoading
from qtui.dayu.theme import MTheme


class LoadingExample(QtWidgets.QWidget, MFieldMixin):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setWindowTitle("Examples for MLoading")
        self._init_ui()

    def _init_ui(self):
        size_lay = QtWidgets.QHBoxLayout()
        size_list = [
            ("Huge", MLoading.huge),
            ("Large", MLoading.large),
            ("Medium", MLoading.medium),
            ("Small", MLoading.small),
            ("Tiny", MLoading.tiny),
        ]
        for label, cls in size_list:
            size_lay.addWidget(MLabel(label))
            size_lay.addWidget(cls())
            size_lay.addSpacing(10)

        color_lay = QtWidgets.QHBoxLayout()
        color_list = [
            ("cyan", "#13c2c2"),
            ("green", "#52c41a"),
            ("magenta", "#eb2f96"),
            ("red", "#f5222d"),
            ("yellow", "#fadb14"),
            ("volcano", "#fa541c"),
        ]
        for label, color in color_list:
            color_lay.addWidget(MLabel(label))
            color_lay.addWidget(MLoading.tiny(color=color))
            color_lay.addSpacing(10)

        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addWidget(MDivider("different size"))
        main_lay.addLayout(size_lay)
        main_lay.addWidget(MDivider("different color"))
        main_lay.addLayout(color_lay)
        main_lay.addWidget(MDivider("loading wrapper"))

        main_lay.addStretch()
        self.setLayout(main_lay)


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = LoadingExample()
        MTheme().apply(test)
        test.show()



from qtpy import QtWidgets

from qtui.dayu.divider import MDivider
from qtui.dayu.page import MPage


class PageExample(QtWidgets.QWidget):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setWindowTitle("Examples for MPage")
        self._init_ui()

    def _init_ui(self):
        page_1 = MPage()
        page_1.set_total(255)
        page_1.sig_page_changed.connect(print)

        page_2 = MPage()
        page_2.set_total(100)

        main_lay = QtWidgets.QVBoxLayout()
        self.setLayout(main_lay)
        main_lay.addWidget(MDivider())
        main_lay.addWidget(page_1)
        main_lay.addWidget(MDivider())
        main_lay.addWidget(page_2)
        main_lay.addStretch()


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = PageExample()
        MTheme().apply(test)
        test.show()



from qtpy import QtWidgets

from qtui.dayu.divider import MDivider
from qtui.dayu.field_mixin import MFieldMixin
from qtui.dayu.tool_button import MToolButton


class ToolButtonExample(QtWidgets.QWidget, MFieldMixin):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setWindowTitle("Examples for MToolButton")
        self._init_ui()

    def _init_ui(self):
        size_lay = QtWidgets.QVBoxLayout()
        sub_lay1 = QtWidgets.QHBoxLayout()
        sub_lay1.addWidget(MToolButton().svg("left_line.svg").icon_only().huge())
        sub_lay1.addWidget(MToolButton().svg("right_line.svg").icon_only().large())
        sub_lay1.addWidget(MToolButton().svg("up_line.svg").icon_only())
        sub_lay1.addWidget(MToolButton().svg("up_line.svg").icon_only().small())
        sub_lay1.addWidget(MToolButton().svg("down_line.svg").icon_only().tiny())
        sub_lay1.addStretch()
        size_lay.addLayout(sub_lay1)

        button2 = MToolButton().svg("detail_line.svg").icon_only()
        button2.setEnabled(False)
        button7 = MToolButton().svg("trash_line.svg").icon_only()
        button7.setCheckable(True)
        state_lay = QtWidgets.QHBoxLayout()
        state_lay.addWidget(button2)
        state_lay.addWidget(button7)
        state_lay.addStretch()

        button_trash = MToolButton().svg("trash_line.svg").text_beside_icon()
        button_trash.setText("Delete")
        button_login = MToolButton().svg("user_line.svg").text_beside_icon()
        button_login.setText("Login")

        button_lay = QtWidgets.QHBoxLayout()
        button_lay.addWidget(button_trash)
        button_lay.addWidget(button_login)

        sub_lay2 = QtWidgets.QHBoxLayout()
        sub_lay2.addWidget(button2)
        sub_lay2.addWidget(button7)

        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addWidget(MDivider("different button_size"))
        main_lay.addLayout(size_lay)
        main_lay.addWidget(MDivider("disabled & checkable"))
        main_lay.addLayout(state_lay)
        main_lay.addWidget(MDivider("type=normal"))
        main_lay.addLayout(button_lay)
        main_lay.addStretch()
        self.setLayout(main_lay)


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = ToolButtonExample()
        MTheme().apply(test)
        test.show()



from typing import cast

from qtpy import QtWidgets

import examples.dayu._mock_data as mock
from qtui.dayu.divider import MDivider
from qtui.dayu.field_mixin import MFieldMixin
from qtui.dayu.item_view import MTreeView
from qtui.dayu.item_view_set import MItemViewSet
from qtui.dayu.push_button import MPushButton


class ItemViewSetExample(QtWidgets.QWidget, MFieldMixin):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self._init_ui()

    def _init_ui(self):
        item_view_set_table = MItemViewSet(view_type=MItemViewSet.TableViewType)
        item_view_set_table.set_header_list(mock.header_list)
        item_view_set_list = MItemViewSet(view_type=MItemViewSet.ListViewType)
        item_view_set_list.set_header_list(mock.header_list)
        item_view_set_tree = MItemViewSet(view_type=MItemViewSet.TreeViewType)
        item_view_set_tree.set_header_list(mock.header_list)
        item_view_set_thumbnail = MItemViewSet(view_type=MItemViewSet.BigViewType)
        item_view_set_thumbnail.set_header_list(mock.header_list)

        item_view_set_search = MItemViewSet(view_type=MItemViewSet.TreeViewType)
        item_view_set_search.set_header_list(mock.header_list)
        item_view_set_search.searchable()
        expand_button = MPushButton("Expand All")
        expand_button.clicked.connect(
            cast(MTreeView, item_view_set_search.item_view).expandAll
        )
        coll_button = MPushButton("Collapse All")
        coll_button.clicked.connect(
            cast(MTreeView, item_view_set_search.item_view).collapseAll
        )
        item_view_set_search.insert_widget(coll_button)
        item_view_set_search.insert_widget(expand_button)

        refresh_button = MPushButton("Refresh Data")
        refresh_button.clicked.connect(self.slot_update_data)
        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addWidget(MDivider("Table View"))
        main_lay.addWidget(refresh_button)
        main_lay.addWidget(item_view_set_table)
        main_lay.addWidget(MDivider("List View"))
        main_lay.addWidget(item_view_set_list)
        main_lay.addWidget(MDivider("Tree View"))
        main_lay.addWidget(item_view_set_tree)
        main_lay.addWidget(MDivider("Big View"))
        main_lay.addWidget(item_view_set_thumbnail)
        main_lay.addWidget(MDivider("With Search line edit"))
        main_lay.addWidget(item_view_set_search)
        main_lay.addStretch()
        self.setLayout(main_lay)

        item_view_set_tree.setup_data(mock.tree_data_list)
        item_view_set_search.setup_data(mock.tree_data_list)
        self.view_list = [
            item_view_set_table,
            item_view_set_list,
            item_view_set_thumbnail,
        ]
        self.slot_update_data()

    def slot_update_data(self):
        for view in self.view_list:
            view.setup_data(mock.data_list)


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = ItemViewSetExample()
        MTheme().apply(test)
        test.show()



import functools
from typing import cast

from qtpy import QtGui, QtWidgets

import examples.dayu._mock_data as mock
from qtui.dayu import utils as dayu_utils
from qtui.dayu.divider import MDivider
from qtui.dayu.field_mixin import MFieldMixin
from qtui.dayu.item_view import MBigView
from qtui.dayu.item_view_set import MItemViewSet
from qtui.dayu.theme import MTheme
from qtui.dayu.tool_button import MToolButton


class ItemViewBigTypeExample(QtWidgets.QWidget, MFieldMixin):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self._init_ui()

    def _init_ui(self):
        item_view_set_thumbnail = MItemViewSet(view_type=MItemViewSet.BigViewType)
        item_view_set_thumbnail.set_header_list(
            [
                {
                    "label": "Name",
                    "key": "name",
                    "searchable": True,
                    "font": lambda x, y: {"underline": True},
                    "icon": lambda x, y: y.get("icon"),
                }
            ]
        )
        add_button = MToolButton().svg("add_line.svg")
        add_button.clicked.connect(
            functools.partial(
                cast(MBigView, item_view_set_thumbnail.item_view).scale_size, 1.1
            )
        )
        minus_button = MToolButton().svg("minus_line.svg")
        minus_button.clicked.connect(
            functools.partial(
                cast(MBigView, item_view_set_thumbnail.item_view).scale_size, 0.8
            )
        )
        button_lay = QtWidgets.QHBoxLayout()
        button_lay.addWidget(minus_button)
        button_lay.addWidget(add_button)
        button_lay.addStretch()

        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addWidget(MDivider("Big View"))
        main_lay.addLayout(button_lay)
        main_lay.addWidget(item_view_set_thumbnail)
        self.setLayout(main_lay)
        for data_dict in mock.data_list:
            icon = QtGui.QIcon(
                dayu_utils.generate_text_pixmap(
                    400,
                    400,
                    f"{data_dict.get('name', '')}_{data_dict.get('sex', '')}",
                    bg_color=MTheme().background_selected_color,
                )
            )
            data_dict.update({"icon": icon})
        item_view_set_thumbnail.setup_data(mock.data_list * 100)


if __name__ == "__main__":
    from qtui.dayu.qt import application

    with application() as app:
        test = ItemViewBigTypeExample()
        MTheme().apply(test)
        test.show()



from qtpy import QtWidgets

from qtui.dayu.divider import MDivider
from qtui.dayu.push_button import MPushButton
from qtui.dayu.text_edit import MTextEdit


class TextEditExample(QtWidgets.QWidget):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self._init_ui()

    def _init_ui(self):
        main_lay = QtWidgets.QVBoxLayout()

        main_lay.addWidget(MDivider("no size grip"))
        main_lay.addWidget(MTextEdit(self))
        main_lay.addWidget(MDivider("size grip"))
        main_lay.addWidget(MTextEdit(self).resizeable())
        main_lay.addWidget(MPushButton("text").primary())

        self.setLayout(main_lay)


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = TextEditExample()
        MTheme().apply(test)
        test.show()



from qtpy import QtCore, QtWidgets

from qtui.dayu.divider import MDivider
from qtui.dayu.field_mixin import MFieldMixin
from qtui.dayu.label import MLabel
from qtui.dayu.push_button import MPushButton


class LabelExample(QtWidgets.QWidget, MFieldMixin):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setWindowTitle("Examples for MLabel")
        self._init_ui()

    def _init_ui(self):
        title_lay = QtWidgets.QGridLayout()
        title_lay.addWidget(MLabel("h1 Level").h1(), 0, 1)
        title_lay.addWidget(MLabel("h2 Level").h2(), 1, 1)
        title_lay.addWidget(MLabel("h3 Level").h3(), 2, 1)
        title_lay.addWidget(MLabel("h4 Level").h4(), 3, 1)

        text_type_lay = QtWidgets.QHBoxLayout()
        text_type_lay.addWidget(MLabel("MLabel: Normal"))
        text_type_lay.addWidget(MLabel("MLabel: Secondary").secondary())
        text_type_lay.addWidget(MLabel("MLabel: Warning").warning())
        text_type_lay.addWidget(MLabel("MLabel: Danger").danger())
        disable_text = MLabel("MLabel: Disabled")
        disable_text.setEnabled(False)
        text_type_lay.addWidget(disable_text)

        text_attr_lay = QtWidgets.QHBoxLayout()
        text_attr_lay.addWidget(MLabel("MLabel: Mark").mark())
        text_attr_lay.addWidget(MLabel("MLabel: Code").code())
        text_attr_lay.addWidget(MLabel("MLabel: Underline").underline())
        text_attr_lay.addWidget(MLabel("MLabel: Delete").delete())
        text_attr_lay.addWidget(MLabel("MLabel: Strong").strong())

        text_mix_lay = QtWidgets.QHBoxLayout()
        text_mix_lay.addWidget(
            MLabel("MLabel: Strong & Underline").strong().underline()
        )
        text_mix_lay.addWidget(MLabel("MLabel: Danger & Delete").danger().delete())
        text_mix_lay.addWidget(MLabel("MLabel: Warning & Strong").warning().strong())
        text_mix_lay.addWidget(MLabel("MLabel: H4 & Mark").h4().mark())

        data_bind_lay = QtWidgets.QHBoxLayout()
        data_bind_label = MLabel()
        button = MPushButton(text="Random An Animal").primary()
        button.clicked.connect(self.slot_change_text)
        data_bind_lay.addWidget(data_bind_label)
        data_bind_lay.addWidget(button)
        data_bind_lay.addStretch()
        self.register_field("show_text", "Guess")
        self.bind("show_text", data_bind_label, "text")

        lay_elide = QtWidgets.QVBoxLayout()
        label_none = MLabel(
            "This is a elide NONE mode label. Ellipsis should NOT appear in the text."
        )
        label_left = MLabel(
            "This is a elide LEFT mode label. "
            "The ellipsis should appear at the beginning of the text. "
            "xiao mao xiao gou xiao ci wei"
        )
        label_left.set_elide_mode(QtCore.Qt.TextElideMode.ElideLeft)
        label_middle = MLabel(
            "This is a elide MIDDLE mode label. "
            "The ellipsis should appear in the middle of the text. "
            "xiao mao xiao gou xiao ci wei"
        )
        label_middle.set_elide_mode(QtCore.Qt.TextElideMode.ElideMiddle)
        label_right = MLabel()
        label_right.setText(
            "This is a elide RIGHT mode label. "
            "The ellipsis should appear at the end of the text. "
            "Some text to fill the line bala bala bala."
        )
        label_right.set_elide_mode(QtCore.Qt.TextElideMode.ElideRight)
        lay_elide.addWidget(label_none)
        lay_elide.addWidget(label_left)
        lay_elide.addWidget(label_middle)
        lay_elide.addWidget(label_right)

        hyper_label_1 = MLabel()
        hyper_label_1.set_link("https://google.com", text="google")
        hyper_label_2 = MLabel()
        hyper_label_2.set_link("https://google.com")
        hyper_label_3 = MLabel()
        hyper_label_3.set_link(
            "https://github.com/phenom-films/dayu_widgets", text="Dayu Widgets"
        )

        hyperlink_lay = QtWidgets.QVBoxLayout()
        hyperlink_lay.addWidget(hyper_label_1)
        hyperlink_lay.addWidget(hyper_label_2)
        hyperlink_lay.addWidget(hyper_label_3)

        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addWidget(MDivider("different level"))
        main_lay.addLayout(title_lay)
        main_lay.addWidget(MDivider("different type"))
        main_lay.addLayout(text_type_lay)
        main_lay.addWidget(MDivider("different property"))
        main_lay.addLayout(text_attr_lay)
        main_lay.addWidget(MDivider("mix"))
        main_lay.addLayout(text_mix_lay)

        main_lay.addWidget(MDivider("elide mode"))
        main_lay.addLayout(lay_elide)
        main_lay.addWidget(MDivider("hyperlink"))
        main_lay.addLayout(hyperlink_lay)
        main_lay.addStretch()
        self.setLayout(main_lay)

    def slot_change_text(self):
        import random

        self.set_field("show_text", random.choice(["Dog", "Cat", "Rabbit", "Cow"]))

    def slot_link_text(self):
        self.set_field("is_link", not self.field("is_link"))


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = LabelExample()
        MTheme().apply(test)
        test.show()



from qtpy import QtCore, QtWidgets

from qtui.dayu.button_group import MToolButtonGroup
from qtui.dayu.divider import MDivider
from qtui.dayu.field_mixin import MFieldMixin
from qtui.dayu.qt import MIcon
from qtui.dayu.theme import MTheme
from qtui.dayu.types import MButtonGroupData


class ToolButtonGroupExample(QtWidgets.QWidget, MFieldMixin):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self._init_ui()

    def _init_ui(self):
        tool_group_h = MToolButtonGroup(size=MTheme().small)
        tool_group_h.set_button_list(["Apple", {"text": "Banana"}, {"text": "Pear"}])
        tool_1_lay = QtWidgets.QHBoxLayout()
        tool_1_lay.addWidget(tool_group_h)
        tool_1_lay.addStretch()

        app_data = [
            MButtonGroupData(text="Maya", icon=MIcon("app-maya.png"), checkable=True),
            MButtonGroupData(text="Nuke", icon=MIcon("app-nuke.png"), checkable=True),
            MButtonGroupData(
                text="Houdini", icon=MIcon("app-houdini.png"), checkable=True
            ),
        ]

        tool_group_v = MToolButtonGroup(
            exclusive=True,
            size=MTheme().small,
            orientation=QtCore.Qt.Orientation.Vertical,
        )
        tool_group_v.set_button_list(app_data)

        tool_group_button_h = MToolButtonGroup()
        tool_group_button_h.set_button_list(app_data)
        tool_2_lay = QtWidgets.QHBoxLayout()
        tool_2_lay.addWidget(tool_group_button_h)
        tool_2_lay.addStretch()

        tool_grp_excl_true = MToolButtonGroup(
            orientation=QtCore.Qt.Orientation.Horizontal, exclusive=True
        )
        tool_grp_excl_true.set_button_list(
            [
                MButtonGroupData(
                    svg="table_view.svg", checkable=True, tooltip="Table View"
                ),
                MButtonGroupData(
                    svg="list_view.svg", checkable=True, tooltip="List View"
                ),
                MButtonGroupData(
                    svg="tree_view.svg", checkable=True, tooltip="Tree View"
                ),
                MButtonGroupData(
                    svg="big_view.svg", checkable=True, tooltip="Big View"
                ),
            ]
        )
        tool_grp_excl_true.set_dayu_checked(0)
        tool_excl_lay = QtWidgets.QHBoxLayout()
        tool_excl_lay.addWidget(tool_grp_excl_true)
        tool_excl_lay.addStretch()

        tool_grp_excl_false = MToolButtonGroup(
            orientation=QtCore.Qt.Orientation.Horizontal, exclusive=False
        )
        tool_grp_excl_false.set_button_list(
            [
                MButtonGroupData(tooltip="bold", svg="bold.svg", checkable=True),
                MButtonGroupData(tooltip="italic", svg="italic.svg", checkable=True),
                MButtonGroupData(
                    tooltip="underline", svg="underline.svg", checkable=True
                ),
            ]
        )
        tool_excl_2_lay = QtWidgets.QHBoxLayout()
        tool_excl_2_lay.addWidget(tool_grp_excl_false)
        tool_excl_2_lay.addStretch()

        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addWidget(MDivider("orientation=Qt.Horizontal "))
        main_lay.addLayout(tool_1_lay)
        main_lay.addWidget(MDivider("orientation=Qt.Vertical"))
        main_lay.addWidget(tool_group_v)
        main_lay.addWidget(MDivider("orientation=Qt.Horizontal"))
        main_lay.addLayout(tool_2_lay)
        main_lay.addWidget(MDivider("checkable=True; exclusive=True"))
        main_lay.addLayout(tool_excl_lay)
        main_lay.addWidget(MDivider("checkable=True; exclusive=False"))
        main_lay.addLayout(tool_excl_2_lay)
        main_lay.addStretch()
        self.setLayout(main_lay)


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = ToolButtonGroupExample()
        MTheme().apply(test)
        test.show()



from qtpy import QtCore, QtWidgets

from qtui.dayu.check_box import MCheckBox
from qtui.dayu.divider import MDivider
from qtui.dayu.field_mixin import MFieldMixin
from qtui.dayu.label import MLabel
from qtui.dayu.push_button import MPushButton
from qtui.dayu.qt import MIcon


class CheckBoxExample(QtWidgets.QWidget, MFieldMixin):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setWindowTitle("Example for MCheckBox")
        grid_lay = QtWidgets.QGridLayout()

        for index, (text, state) in enumerate(
            [
                ("Unchecked", QtCore.Qt.CheckState.Unchecked),
                ("Checked", QtCore.Qt.CheckState.Checked),
                ("Partially", QtCore.Qt.CheckState.PartiallyChecked),
            ]
        ):
            check_box_normal = MCheckBox(text)
            check_box_normal.setCheckState(state)

            check_box_disabled = MCheckBox(text)
            check_box_disabled.setCheckState(state)
            check_box_disabled.setEnabled(False)

            grid_lay.addWidget(check_box_normal, 0, index)
            grid_lay.addWidget(check_box_disabled, 1, index)

        icon_lay = QtWidgets.QHBoxLayout()
        for text, icon in [
            ("Maya", MIcon("app-maya.png")),
            ("Nuke", MIcon("app-nuke.png")),
            ("Houdini", MIcon("app-houdini.png")),
        ]:
            check_box_icon = MCheckBox(text)
            check_box_icon.setIcon(icon)
            icon_lay.addWidget(check_box_icon)

        check_box_bind = MCheckBox("Data Bind")
        label = MLabel()
        button = MPushButton(text="Change State")
        button.clicked.connect(
            lambda: self.set_field("checked", not self.field("checked"))
        )
        self.register_field("checked", True)
        self.register_field(
            "checked_text",
            lambda: "Yes!" if self.field("checked") else "No!!",
        )
        self.bind("checked", check_box_bind, "checked", signal="stateChanged")
        self.bind("checked_text", label, "text")

        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addWidget(MDivider("Basic"))
        main_lay.addLayout(grid_lay)
        main_lay.addWidget(MDivider("Icon"))
        main_lay.addLayout(icon_lay)
        main_lay.addWidget(MDivider("Data Bind"))
        main_lay.addWidget(check_box_bind)
        main_lay.addWidget(label)
        main_lay.addWidget(button)
        main_lay.addStretch()
        self.setLayout(main_lay)


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = CheckBoxExample()
        MTheme().apply(test)
        test.show()



"""
MAlert class.
"""

import functools
from enum import StrEnum

from qtpy import QtCore, QtWidgets

from .avatar import MAvatar
from .label import MLabel
from .mixin import property_mixin
from .qt import MPixmap, get_scale_factor
from .theme import MTheme
from .tool_button import MToolButton
from .widget import MWidget


@property_mixin
class MAlert(QtWidgets.QWidget, MWidget):  # pyright: ignore[reportGeneralTypeIssues]
    """
    Alert component for feedback.

    Property:
        dayu_type: The feedback type with different color container.
        dayu_text: The feedback string showed in container.
    """

    class AlertType(StrEnum):
        Info = "info"
        Success = "success"
        Warning = "warning"
        Error = "error"

    def __init__(
        self,
        text: str = "",
        parent: QtWidgets.QWidget | None = None,
        flags: QtCore.Qt.WindowType = QtCore.Qt.WindowType.Widget,
    ):
        super().__init__(parent, flags)
        self.setAttribute(QtCore.Qt.WidgetAttribute.WA_StyledBackground)

        self._icon_label = MAvatar.tiny()
        self._content_label = MLabel().secondary()
        self._close_button = MToolButton().svg("close_line.svg").tiny().icon_only()
        self._close_button.clicked.connect(functools.partial(self.setVisible, False))

        scale_x, _ = get_scale_factor()
        margin = int(8 * scale_x)

        self._main_layout = QtWidgets.QHBoxLayout()
        self._main_layout.setContentsMargins(margin, margin, margin, margin)
        self._main_layout.addWidget(self._icon_label)
        self._main_layout.addWidget(self._content_label)
        self._main_layout.addStretch()
        self._main_layout.addWidget(self._close_button)

        self.setLayout(self._main_layout)
        self.set_show_icon(True)
        self.set_closable(False)

        self._dayu_text: str
        self.set_dayu_type(MAlert.AlertType.Info)
        self.set_dayu_text(text)

    def set_closable(self, closable: bool):
        """Display the close icon button or not."""
        self._close_button.setVisible(closable)

    def set_show_icon(self, show_icon: bool):
        """Display the information type icon or not."""
        self._icon_label.setVisible(show_icon)

    def _set_dayu_text(self):
        self._content_label.setText(self._dayu_text)
        self.setVisible(bool(self._dayu_text))

    def set_dayu_text(self, value: str):
        """Set the feedback content."""

        self._dayu_text = value
        self._set_dayu_text()

    def _set_dayu_type(self):
        self._icon_label.set_dayu_image(
            MPixmap(
                f"{self._dayu_type}_fill.svg",
                MTheme().get_color(self._dayu_type),
            )
        )
        self.on_style_changed()

    def set_dayu_type(self, value: str):
        """Set feedback type."""

        if value != self._dayu_type:
            self._dayu_type = value
            self._icon_label.set_dayu_image(
                MPixmap(
                    f"{self._dayu_type}_fill.svg",
                    MTheme().get_color(self._dayu_type),
                )
            )
            self.on_style_changed()

    def get_dayu_type(self):
        """
        Get MAlert feedback type.
        :return: str
        """
        return self._dayu_type

    def get_dayu_text(self):
        """
        Get MAlert feedback message.
        :return: str
        """
        return self._dayu_text

    dayu_text = QtCore.Property(str, get_dayu_text, set_dayu_text)
    dayu_type = QtCore.Property(str, get_dayu_type, set_dayu_type)

    def info(self):
        """Set MAlert to InfoType"""
        self.set_dayu_type(MAlert.AlertType.Info)
        return self

    def success(self):
        """Set MAlert to SuccessType"""
        self.set_dayu_type(MAlert.AlertType.Success)
        return self

    def warning(self):
        """Set MAlert to  WarningType"""
        self.set_dayu_type(MAlert.AlertType.Warning)
        return self

    def error(self):
        """Set MAlert to ErrorType"""
        self.set_dayu_type(MAlert.AlertType.Error)
        return self

    def closable(self):
        """Set MAlert closebale is True"""
        self.set_closable(True)
        return self



from typing import Any

from qtpy import QtCore, QtWidgets

from .mixin import property_mixin
from .theme import MTheme
from .utils import str_to_qbytearray


@property_mixin  # pyright: ignore[reportArgumentType, reportUntypedClassDecorator]
class MCompleter(QtWidgets.QCompleter):
    ITEM_HEIGHT = 28

    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setProperty("animatable", True)

        if not (popup := self.popup()):
            return

        MTheme().apply(popup)

        self._opacity_anim = QtCore.QPropertyAnimation(
            popup, str_to_qbytearray("windowOpacity")
        )
        self.setProperty("anim_opacity_duration", 300)
        self.setProperty("anim_opacity_curve", "OutCubic")
        self.setProperty("anim_opacity_start", 0)
        self.setProperty("anim_opacity_end", 1)

        self._size_anim = QtCore.QPropertyAnimation(popup, str_to_qbytearray("size"))
        self.setProperty("anim_size_duration", 300)
        self.setProperty("anim_size_curve", "OutCubic")

        popup.installEventFilter(self)

    def _set_anim_opacity_duration(self, value: int):
        self._opacity_anim.setDuration(value)

    def _set_anim_opacity_curve(self, value: str):
        curve = getattr(QtCore.QEasingCurve, value, None)
        assert curve, "invalid QEasingCurve"
        self._opacity_anim.setEasingCurve(curve)

    def _set_anim_opacity_start(self, value: Any):
        self._opacity_anim.setStartValue(value)

    def _set_anim_opacity_end(self, value: Any):
        self._opacity_anim.setEndValue(value)

    def _set_anim_size_duration(self, value: Any):
        self._size_anim.setDuration(value)

    def _set_anim_size_curve(self, value: str):
        curve = getattr(QtCore.QEasingCurve, value, None)
        assert curve, "invalid QEasingCurve"
        self._size_anim.setEasingCurve(curve)

    def _set_anim_size_start(self, value: Any):
        self._size_anim.setStartValue(value)

    def _set_anim_size_end(self, value: Any):
        self._size_anim.setEndValue(value)

    def init_size(self):
        if not (popup := self.popup()):
            return

        model = popup.model()

        width = self.widget().width()
        max_height = popup.sizeHint().height()
        item_height = model.data(model.index(0, 0), QtCore.Qt.ItemDataRole.SizeHintRole)
        height = (item_height or self.ITEM_HEIGHT) * model.rowCount()
        height = height if height < max_height else max_height

        start_size = self.property("anim_size_start")
        start_size = start_size if start_size else QtCore.QSize(0, 0)
        end_size = self.property("anim_size_end")
        end_size = end_size if end_size else QtCore.QSize(width, height)
        self._size_anim.setStartValue(start_size)
        self._size_anim.setEndValue(end_size)

    def start_anim(self):
        self.init_size()
        self._opacity_anim.start()
        self._size_anim.start()

    def eventFilter(self, widget: QtWidgets.QWidget, event: QtCore.QEvent):  # noqa: N802  # pyright: ignore[reportIncompatibleMethodOverride]
        if event.type() == QtCore.QEvent.Type.Show and self.property("animatable"):
            self.start_anim()
        return super().eventFilter(widget, event)



from qtpy import QtWidgets

from qtui.dayu.avatar import MAvatar
from qtui.dayu.badge import MBadge
from qtui.dayu.combo_box import MComboBox
from qtui.dayu.divider import MDivider
from qtui.dayu.field_mixin import MFieldMixin
from qtui.dayu.label import MLabel
from qtui.dayu.menu import MMenu
from qtui.dayu.qt import MPixmap
from qtui.dayu.spin_box import MSpinBox
from qtui.dayu.theme import MTheme
from qtui.dayu.tool_button import MToolButton


class BadgeExample(QtWidgets.QWidget, MFieldMixin):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setWindowTitle("Examples for MBadge")
        self._init_ui()

    def _init_ui(self):
        standalone_lay = QtWidgets.QHBoxLayout()
        standalone_lay.addWidget(MBadge.count(0))
        standalone_lay.addWidget(MBadge.count(20))
        standalone_lay.addWidget(MBadge.count(100))
        standalone_lay.addWidget(MBadge.dot(True))
        standalone_lay.addWidget(MBadge.text("new"))
        standalone_lay.addStretch()

        button = MToolButton().svg("trash_line.svg")
        avatar = MAvatar.large(MPixmap("avatar.png"))
        button_alert = MToolButton().svg("alert_fill.svg").large()
        badge_1 = MBadge.dot(True, widget=button)
        badge_2 = MBadge.dot(True, widget=avatar)
        badge_3 = MBadge.dot(True, widget=button_alert)
        button.clicked.connect(lambda: badge_1.set_dayu_dot(not badge_1.get_dayu_dot()))

        spin_box = MSpinBox()
        spin_box.setRange(0, 9999)
        spin_box.valueChanged.connect(badge_3.set_dayu_count)
        spin_box.setValue(1)

        self.register_field("button1_selected", "Beijing")
        menu1 = MMenu(parent=self)
        menu1.set_data(["Beijing", "Shanghai", "Guangzhou", "Shenzhen"])
        select1 = MComboBox()
        select1.set_menu(menu1)
        self.bind("button1_selected", select1, "value", signal="value_changed")

        badge_hot = MBadge.text("hot", widget=MLabel("Your ideal city  "))

        sub_lay1 = QtWidgets.QHBoxLayout()
        sub_lay1.addWidget(badge_1)
        sub_lay1.addWidget(badge_2)
        sub_lay1.addWidget(badge_3)
        sub_lay1.addStretch()

        sub_lay2 = QtWidgets.QHBoxLayout()
        sub_lay2.addWidget(badge_hot)
        sub_lay2.addWidget(select1)
        sub_lay2.addStretch()

        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addWidget(MDivider("use standalone"))
        main_lay.addLayout(standalone_lay)
        main_lay.addWidget(MDivider("different type"))
        main_lay.addLayout(sub_lay1)
        main_lay.addWidget(spin_box)
        main_lay.addWidget(MDivider("different type"))
        main_lay.addLayout(sub_lay2)
        main_lay.addStretch()
        self.setLayout(main_lay)


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = BadgeExample()
        MTheme().apply(test)
        test.show()



import functools
from typing import Any

from qtpy import QtCore, QtWidgets

import examples.dayu._mock_data as mock
from qtui.dayu.divider import MDivider
from qtui.dayu.field_mixin import MFieldMixin
from qtui.dayu.item_model import MSortFilterModel, MTableModel
from qtui.dayu.item_view import MTableView
from qtui.dayu.line_edit import MLineEdit
from qtui.dayu.loading import MLoadingWrapper
from qtui.dayu.push_button import MPushButton
from qtui.dayu.theme import MTheme


class MFetchDataThread(QtCore.QThread):
    def __init__(self, parent: QtCore.QObject | None = None):
        super().__init__(parent)

    def run(self, *args: Any, **kwargs: Any):
        import time

        time.sleep(4)


class TableViewExample(QtWidgets.QWidget, MFieldMixin):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self._init_ui()

    def _init_ui(self):
        model_1 = MTableModel()
        model_1.set_header_list(mock.header_list)
        model_sort = MSortFilterModel()
        model_sort.setSourceModel(model_1)

        table_small = MTableView(size=MTheme().small, show_row_count=True)
        table_grid = MTableView(size=MTheme().small, show_row_count=True)
        table_grid.setShowGrid(True)
        table_default = MTableView(size=MTheme().medium, show_row_count=True)
        thread = MFetchDataThread(self)

        self.loading_wrapper = MLoadingWrapper(widget=table_default, loading=False)
        thread.started.connect(
            functools.partial(self.loading_wrapper.set_dayu_loading, True)
        )
        thread.finished.connect(
            functools.partial(self.loading_wrapper.set_dayu_loading, False)
        )
        thread.finished.connect(functools.partial(table_default.setModel, model_sort))
        button = MPushButton(text="Get Data: 4s")
        button.clicked.connect(thread.start)
        switch_lay = QtWidgets.QHBoxLayout()
        switch_lay.addWidget(button)
        switch_lay.addStretch()
        table_large = MTableView(size=MTheme().large, show_row_count=False)

        table_small.setModel(model_sort)
        table_grid.setModel(model_sort)
        table_large.setModel(model_sort)
        model_sort.set_header_list(mock.header_list)
        table_small.set_header_list(mock.header_list)
        table_grid.set_header_list(mock.header_list)
        table_default.set_header_list(mock.header_list)
        table_large.set_header_list(mock.header_list)
        model_1.set_data_list(mock.data_list)

        line_edit = MLineEdit().search_with_close().small()
        line_edit.textChanged.connect(model_sort.set_search_pattern)

        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addWidget(line_edit)
        main_lay.addWidget(MDivider("Small Size"))
        main_lay.addWidget(table_small)
        main_lay.addWidget(MDivider("Default Size"))
        main_lay.addLayout(switch_lay)
        main_lay.addWidget(self.loading_wrapper)
        main_lay.addWidget(MDivider("Large Size (Hide Row Count)"))
        main_lay.addWidget(table_large)
        main_lay.addWidget(MDivider("With Grid"))
        main_lay.addWidget(table_grid)
        main_lay.addStretch()
        self.setLayout(main_lay)


if __name__ == "__main__":
    from qtui.dayu.qt import application

    with application() as app:
        test = TableViewExample()
        MTheme().apply(test)
        test.show()



from functools import partial

from qtpy import QtCore, QtWidgets

from .mixin import property_mixin
from .theme import MTheme


@property_mixin
class MSplitter(QtWidgets.QSplitter):
    def __init__(
        self,
        orientation: QtCore.Qt.Orientation = QtCore.Qt.Orientation.Horizontal,
        parent: QtWidgets.QWidget | None = None,
    ):
        super().__init__(orientation, parent=parent)
        self.setHandleWidth(10)
        self.setProperty("animatable", True)
        self.setProperty("default_size", 100)
        self.setProperty("anim_move_duration", 300)
        MTheme().apply(self)

    def slot_splitter_click(self, index: int, first: bool = True):
        size_list = self.sizes()
        prev = index - 1
        prev_size = size_list[prev]
        next_size = size_list[index]
        default_size = self.property("default_size")
        if not prev_size:
            size_list[prev] = default_size
            size_list[index] -= default_size
        elif not next_size:
            size_list[index] = default_size
            size_list[prev] -= default_size
        else:
            if first:
                size_list[index] += prev_size
                size_list[prev] = 0
            else:
                size_list[prev] += next_size
                size_list[index] = 0

        if self.property("animatable"):
            anim = QtCore.QVariantAnimation(self)

            def anim_size(index: int, size_list: list[int], v: int):
                size_list[index - 1] += size_list[index] - v
                size_list[index] = v
                self.setSizes(size_list)

            anim.valueChanged.connect(partial(anim_size, index, size_list))
            anim.setDuration(self.property("anim_move_duration"))
            anim.setStartValue(next_size)
            anim.setEndValue(size_list[index])
            anim.start()
        else:
            self.setSizes(size_list)

    def createHandle(self) -> QtWidgets.QSplitterHandle:  # noqa: N802
        count = self.count()

        orient = self.orientation()
        is_horizontal = orient is QtCore.Qt.Orientation.Horizontal
        handle = QtWidgets.QSplitterHandle(orient, self)

        # NOTES: double click average size
        handle.mouseDoubleClickEvent = lambda e: self.setSizes(
            [1 for _ in range(self.count())]
        )

        layout = QtWidgets.QVBoxLayout() if is_horizontal else QtWidgets.QHBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        handle.setLayout(layout)

        button = QtWidgets.QToolButton(handle)
        button.setArrowType(
            QtCore.Qt.ArrowType.LeftArrow
            if is_horizontal
            else QtCore.Qt.ArrowType.UpArrow
        )
        button.clicked.connect(lambda: self.slot_splitter_click(count, True))
        layout.addWidget(button)
        button = QtWidgets.QToolButton(handle)
        arrow = (
            QtCore.Qt.ArrowType.RightArrow
            if is_horizontal
            else QtCore.Qt.ArrowType.DownArrow
        )
        button.setArrowType(arrow)
        button.clicked.connect(lambda: self.slot_splitter_click(count, False))
        layout.addWidget(button)

        return handle



from typing import TypeVar, cast

from qtpy import QtCore, QtWidgets

from .item_model import MSortFilterModel, MTableModel
from .item_view import MAbstractView, MBigView, MListView, MTableView, MTreeView
from .line_edit import MLineEdit
from .tool_button import MToolButton
from .types import HeaderData, ModelData

T = TypeVar("T", bound=MTableView | MBigView | MTreeView | MListView)


class MItemViewSet(QtWidgets.QWidget):
    sig_double_clicked = QtCore.Signal(QtCore.QModelIndex)
    sig_left_clicked = QtCore.Signal(QtCore.QModelIndex)
    TableViewType = MTableView
    BigViewType = MBigView
    TreeViewType = MTreeView
    ListViewType = MListView

    def __init__(
        self,
        view_type: type[T] | None = None,
        parent: QtWidgets.QWidget | None = None,
    ):
        super().__init__(parent)
        self.main_lay = QtWidgets.QVBoxLayout()
        self.main_lay.setSpacing(5)
        self.main_lay.setContentsMargins(0, 0, 0, 0)

        self.sort_filter_model = MSortFilterModel()
        self.source_model = MTableModel()
        self.sort_filter_model.setSourceModel(self.source_model)
        view_class = view_type or MItemViewSet.TableViewType
        self.item_view = cast(MAbstractView, view_class())
        self.item_view.doubleClicked.connect(self.sig_double_clicked)
        self.item_view.pressed.connect(self.slot_left_clicked)
        self.item_view.setModel(self.sort_filter_model)

        self._search_line_edit = MLineEdit().search_with_close().small()
        self._search_attr_button = (
            MToolButton().icon_only().svg("down_fill.svg").small()
        )
        self._search_line_edit.set_prefix_widget(self._search_attr_button)
        self._search_line_edit.textChanged.connect(
            self.sort_filter_model.set_search_pattern
        )
        self._search_line_edit.setVisible(False)
        self._search_lay = QtWidgets.QHBoxLayout()
        self._search_lay.setContentsMargins(0, 0, 0, 0)
        self._search_lay.addStretch()
        self._search_lay.addWidget(self._search_line_edit)

        self.main_lay.addLayout(self._search_lay)
        self.main_lay.addWidget(self.item_view)
        self.setLayout(self.main_lay)

    @QtCore.Slot(QtCore.QModelIndex)
    def slot_left_clicked(self, start_index: QtCore.QModelIndex):
        button = QtWidgets.QApplication.mouseButtons()
        if button == QtCore.Qt.MouseButton.LeftButton:
            real_index = self.sort_filter_model.mapToSource(start_index)
            self.sig_left_clicked.emit(real_index)

    def set_header_list(self, header_list: list[HeaderData]):
        self.source_model.set_header_list(header_list)
        self.sort_filter_model.set_header_list(header_list)
        self.sort_filter_model.setSourceModel(self.source_model)
        self.source_model.clear()
        self.item_view.set_header_list(header_list)

    @QtCore.Slot()
    def setup_data(self, data_list: list[ModelData] | None):
        self.source_model.clear()
        if data_list:
            self.source_model.set_data_list(data_list)

    def get_data(self) -> list[ModelData]:
        return self.source_model.get_data_list()

    def searchable(self):
        """Enable search line edit visible."""
        self._search_line_edit.setVisible(True)
        return self

    def insert_widget(self, widget: QtWidgets.QWidget):
        """Use can insert extra widget into search layout."""
        self._search_lay.insertWidget(0, widget)



"""MMessage"""

from typing import ClassVar

from qtpy import QtCore, QtWidgets

from .avatar import MAvatar
from .label import MLabel
from .loading import MLoading
from .qt import MPixmap
from .theme import MTheme
from .tool_button import MToolButton


class MMessage(QtWidgets.QWidget):
    """
    Display global messages as feedback in response to user operations.
    """

    InfoType = "info"
    SuccessType = "success"
    WarningType = "warning"
    ErrorType = "error"
    LoadingType = "loading"

    default_config: ClassVar[dict[str, int]] = {"duration": 2, "top": 24}

    sig_closed = QtCore.Signal()

    def __init__(
        self,
        text: str,
        duration: int | None = None,
        dayu_type: str | None = None,
        closable: bool = False,
        parent: QtWidgets.QWidget | None = None,
    ):
        super().__init__(parent)
        self.setObjectName("message")
        self.setWindowFlags(
            QtCore.Qt.WindowType.FramelessWindowHint | QtCore.Qt.WindowType.Dialog
        )
        self.setAttribute(QtCore.Qt.WidgetAttribute.WA_DeleteOnClose)

        if dayu_type == MMessage.LoadingType:
            _icon_label = MLoading.tiny()
        else:
            _icon_label = MAvatar.tiny()
            current_type = dayu_type or MMessage.InfoType
            _icon_label.set_dayu_image(
                MPixmap(
                    f"{current_type}_fill.svg",
                    vars(MTheme()).get(current_type + "_color"),
                )
            )

        self._content_label = MLabel(parent=self)
        self._content_label.setText(text)

        self._close_button = (
            MToolButton(parent=self).icon_only().svg("close_line.svg").tiny()
        )
        self._close_button.clicked.connect(self.close)
        self._close_button.setVisible(closable or False)

        self._main_lay = QtWidgets.QHBoxLayout()
        self._main_lay.addWidget(_icon_label)
        self._main_lay.addWidget(self._content_label)
        self._main_lay.addStretch()
        self._main_lay.addWidget(self._close_button)
        self.setLayout(self._main_lay)

        _close_timer = QtCore.QTimer(self)
        _close_timer.setSingleShot(True)
        _close_timer.timeout.connect(self.close)
        _close_timer.timeout.connect(self.sig_closed)
        _close_timer.setInterval((duration or self.default_config["duration"]) * 1000)

        _ani_timer = QtCore.QTimer(self)
        _ani_timer.timeout.connect(self._fade_out)
        _ani_timer.setInterval(
            (duration or self.default_config["duration"]) * 1000 - 300
        )

        _close_timer.start()
        _ani_timer.start()

        self._pos_ani = QtCore.QPropertyAnimation(self)
        self._pos_ani.setTargetObject(self)
        self._pos_ani.setEasingCurve(QtCore.QEasingCurve.Type.OutCubic)
        self._pos_ani.setDuration(300)
        self._pos_ani.setPropertyName(b"pos")

        self._opacity_ani = QtCore.QPropertyAnimation()
        self._opacity_ani.setTargetObject(self)
        self._opacity_ani.setDuration(300)
        self._opacity_ani.setEasingCurve(QtCore.QEasingCurve.Type.OutCubic)
        self._opacity_ani.setPropertyName(b"windowOpacity")
        self._opacity_ani.setStartValue(0.0)
        self._opacity_ani.setEndValue(1.0)

        self._set_proper_position(parent)
        self._fade_int()

    def _fade_out(self):
        self._pos_ani.setDirection(QtCore.QAbstractAnimation.Direction.Backward)
        self._pos_ani.start()
        self._opacity_ani.setDirection(QtCore.QAbstractAnimation.Direction.Backward)
        self._opacity_ani.start()

    def _fade_int(self):
        self._pos_ani.start()
        self._opacity_ani.start()

    def _set_proper_position(self, parent: QtWidgets.QWidget | None):
        if parent is None:
            return
        parent_geo = parent.geometry()
        if parent.isWindowType() or parent in QtWidgets.QApplication.topLevelWidgets():
            pos = parent_geo.topLeft()
        else:
            pos = parent.mapToGlobal(parent_geo.topLeft())
        offset = 0
        for child in parent.children():
            if isinstance(child, MMessage) and child.isVisible():
                offset = max(offset, child.y())
        base = pos.y() + MMessage.default_config["top"]
        target_x = pos.x() + int(parent_geo.width() / 2) - 100
        target_y = (offset + 50) if offset else base
        self._pos_ani.setStartValue(QtCore.QPoint(target_x, target_y - 40))
        self._pos_ani.setEndValue(QtCore.QPoint(target_x, target_y))

    @classmethod
    def info(
        cls,
        text: str,
        parent: QtWidgets.QWidget,
        duration: int | None = None,
        closable: bool = False,
    ):
        """Show a normal message"""
        inst = cls(
            text,
            dayu_type=MMessage.InfoType,
            duration=duration,
            closable=closable,
            parent=parent,
        )
        inst.show()
        return inst

    @classmethod
    def success(
        cls,
        text: str,
        parent: QtWidgets.QWidget,
        duration: int | None = None,
        closable: bool = False,
    ):
        """Show a success message"""
        inst = cls(
            text,
            dayu_type=MMessage.SuccessType,
            duration=duration,
            closable=closable,
            parent=parent,
        )

        inst.show()
        return inst

    @classmethod
    def warning(
        cls,
        text: str,
        parent: QtWidgets.QWidget,
        duration: int | None = None,
        closable: bool = False,
    ):
        """Show a warning message"""
        inst = cls(
            text,
            dayu_type=MMessage.WarningType,
            duration=duration,
            closable=closable,
            parent=parent,
        )
        inst.show()
        return inst

    @classmethod
    def error(
        cls,
        text: str,
        parent: QtWidgets.QWidget,
        duration: int | None = None,
        closable: bool = False,
    ):
        """Show an error message"""
        inst = cls(
            text,
            dayu_type=MMessage.ErrorType,
            duration=duration,
            closable=closable,
            parent=parent,
        )
        inst.show()
        return inst

    @classmethod
    def loading(cls, text: str, parent: QtWidgets.QWidget):
        """Show a message with loading animation"""
        inst = cls(text, dayu_type=MMessage.LoadingType, parent=parent)
        inst.show()
        return inst

    @classmethod
    def config(cls, duration: int | None = None, top: int | None = None):
        """
        Config the global MMessage duration and top setting.
        :param duration: int (unit is second)
        :param top: int (unit is px)
        :return: None
        """
        if duration is not None:
            cls.default_config["duration"] = duration
        if top is not None:
            cls.default_config["top"] = top



"""
MSwitch
"""

from __future__ import annotations

from qtpy import QtCore, QtWidgets

from .mixin import cursor_mixin
from .theme import MTheme


@cursor_mixin
class MSwitch(QtWidgets.QRadioButton):
    """
    Switching Selector.

    Property:
        dayu_size: the size of switch widget. int
    """

    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self._dayu_size: int = MTheme().default_size
        self.setAutoExclusive(False)

    def minimumSizeHint(self) -> QtCore.QSize:  # noqa: N802
        """
        Override the QRadioButton minimum size hint. We don't need the text space.
        :return:
        """
        height = self._dayu_size * 1.2
        return QtCore.QSize(int(height), int(height / 2))

    def get_dayu_size(self) -> int:
        """
        Get the switch size.
        :return: int
        """
        return self._dayu_size

    def set_dayu_size(self, value: int) -> None:
        """
        Set the switch size.
        :param value: int
        :return: None
        """
        self._dayu_size = value
        self.style().polish(self)

    dayu_size = QtCore.Property(int, get_dayu_size, set_dayu_size)

    def huge(self) -> MSwitch:
        """Set MSwitch to huge size"""
        self.set_dayu_size(MTheme().huge)
        return self

    def large(self):
        """Set MSwitch to large size"""
        self.set_dayu_size(MTheme().large)
        return self

    def medium(self):
        """Set MSwitch to medium size"""
        self.set_dayu_size(MTheme().medium)
        return self

    def small(self):
        """Set MSwitch to small size"""
        self.set_dayu_size(MTheme().small)
        return self

    def tiny(self):
        """Set MSwitch to tiny size"""
        self.set_dayu_size(MTheme().tiny)
        return self



import re
from enum import IntEnum, StrEnum

from qtpy import QtCore, QtWidgets

from .theme import MTheme
from .widget import MWidget


class MLabel(QtWidgets.QLabel, MWidget):
    """
    Display title in different level.
    """

    class LabelLevel(IntEnum):
        H1 = 1
        H2 = 2
        H3 = 3
        H4 = 4

    class LabelType(StrEnum):
        Secondary = "secondary"
        Warning = "warning"
        Danger = "danger"

    dayu_text_changed = QtCore.Signal(str)

    def __init__(
        self,
        text: str = "",
        parent: QtWidgets.QWidget | None = None,
        flags: QtCore.Qt.WindowType = QtCore.Qt.WindowType.Widget,
    ):
        super().__init__(text, parent, flags)
        self.setTextInteractionFlags(
            QtCore.Qt.TextInteractionFlag.TextBrowserInteraction
            | QtCore.Qt.TextInteractionFlag.LinksAccessibleByMouse
        )
        self.setSizePolicy(
            QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Minimum
        )

        self._dayu_text: str = text
        self._dayu_level: int = 0
        self._dayu_underline: bool = False
        self._dayu_delete: bool = False
        self._dayu_strong: bool = False
        self._dayu_mark: bool = False
        self._dayu_code: bool = False
        self._elide_mode = QtCore.Qt.TextElideMode.ElideNone

        self.dayu_text_changed.connect(self.on_dayu_text_changed)

    def get_dayu_text(self) -> str:
        return self._dayu_text

    def set_dayu_text(self, value: str) -> None:
        if value != self._dayu_text:
            self._dayu_text = value
            self.setText(value)
            self.dayu_text_changed.emit(value)

    def get_dayu_level(self) -> int:
        """Get MLabel level."""
        return self._dayu_level

    def set_dayu_level(self, value: int) -> None:
        """Set MLabel level"""
        if value != self._dayu_level:
            self._dayu_level = value
            self.on_style_changed()

    def set_dayu_underline(self, value: bool) -> None:
        """Set MLabel underline style."""
        if value != self._dayu_underline:
            self._dayu_underline = value
            self.on_style_changed()

    def get_dayu_underline(self) -> bool:
        return self._dayu_underline

    def set_dayu_delete(self, value: bool) -> None:
        """Set MLabel a delete line style."""
        if value != self._dayu_delete:
            self._dayu_delete = value
            self.on_style_changed()

    def get_dayu_delete(self) -> bool:
        return self._dayu_delete

    def set_dayu_strong(self, value: bool) -> None:
        """Set MLabel bold style."""
        if value != self._dayu_strong:
            self._dayu_strong = value
            self.on_style_changed()

    def get_dayu_strong(self) -> bool:
        return self._dayu_strong

    def set_dayu_mark(self, value: bool) -> None:
        """Set MLabel mark style."""
        if value != self._dayu_mark:
            self._dayu_mark = value
            self.on_style_changed()

    def get_dayu_mark(self) -> bool:
        return self._dayu_mark

    def set_dayu_code(self, value: bool) -> None:
        """Set MLabel code style."""
        if value != self._dayu_code:
            self._dayu_code = value
            self.on_style_changed()

    def get_dayu_code(self) -> bool:
        return self._dayu_code

    def get_elide_mode(self) -> QtCore.Qt.TextElideMode:
        return self._elide_mode

    def set_elide_mode(self, value: QtCore.Qt.TextElideMode) -> None:
        """Set MLabel elide mode.
        Only accepted Qt.ElideLeft/Qt.ElideMiddle/Qt.ElideRight/Qt.ElideNone"""
        self._elide_mode = value
        self._update_elided_text()

    dayu_text = QtCore.Property(
        str,
        get_dayu_text,
        set_dayu_text,
        notify=dayu_text_changed,
    )
    dayu_level = QtCore.Property(
        int,
        get_dayu_level,
        set_dayu_level,
    )
    dayu_underline = QtCore.Property(
        bool,
        get_dayu_underline,
        set_dayu_underline,
    )
    dayu_delete = QtCore.Property(
        bool,
        get_dayu_delete,
        set_dayu_delete,
    )
    dayu_strong = QtCore.Property(
        bool,
        get_dayu_strong,
        set_dayu_strong,
    )
    dayu_mark = QtCore.Property(
        bool,
        get_dayu_mark,
        set_dayu_mark,
    )
    dayu_code = QtCore.Property(
        bool,
        get_dayu_code,
        set_dayu_code,
    )
    dayu_elide_mod = QtCore.Property(
        QtCore.Qt.TextElideMode,
        get_dayu_code,
        set_dayu_code,
    )

    def minimumSizeHint(self):  # noqa: N802
        return QtCore.QSize(1, self.fontMetrics().height())

    def setText(self, text: str):  # noqa: N802
        """
        Overridden base method to set the text on the label

        :param text:    The text to set on the label
        """
        super().setText(text)
        self.set_dayu_text(text)
        self._update_elided_text()
        self.setToolTip(text)

    def set_link(self, href: str, text: str = "") -> None:
        """

        :param href: The href attr of a tag
        :param text: The a tag text content
        """
        link_style = MTheme().hyperlink_style
        self.setText(f'{link_style}<a href="{href}">{text or href}</a>')
        self.setOpenExternalLinks(True)

    def _update_elided_text(self):
        """
        Update the elided text on the label
        """
        _font_metrics = self.fontMetrics()
        text = self.property("text")
        text = text if text else ""

        is_html = bool(re.search(r"<[^>]+>", text))

        if is_html:
            super().setText(text)
        else:
            _elided_text = _font_metrics.elidedText(
                text, self._elide_mode, self.width() - 2 * 2
            )
            super().setText(_elided_text)

    def resizeEvent(self, event: QtCore.QEvent):  # noqa: N802
        """
        Overridden base method called when the widget is resized.

        :param event:    The resize event
        """
        self._update_elided_text()

    def h1(self):
        """Set QLabel with h1 type."""
        self.set_dayu_level(MLabel.LabelLevel.H1)
        return self

    def h2(self):
        """Set QLabel with h2 type."""
        self.set_dayu_level(MLabel.LabelLevel.H2)
        return self

    def h3(self):
        """Set QLabel with h3 type."""
        self.set_dayu_level(MLabel.LabelLevel.H3)
        return self

    def h4(self):
        """Set QLabel with h4 type."""
        self.set_dayu_level(MLabel.LabelLevel.H4)
        return self

    def secondary(self):
        """Set QLabel with secondary type."""
        self.set_dayu_type(MLabel.LabelType.Secondary)
        return self

    def warning(self):
        """Set QLabel with warning type."""
        self.set_dayu_type(MLabel.LabelType.Warning)
        return self

    def danger(self):
        """Set QLabel with danger type."""
        self.set_dayu_type(MLabel.LabelType.Danger)
        return self

    def strong(self):
        """Set QLabel with strong style."""
        self.set_dayu_strong(True)
        return self

    def mark(self):
        """Set QLabel with mark style."""
        self.set_dayu_mark(True)
        return self

    def code(self):
        """Set QLabel with code style."""
        self.set_dayu_code(True)
        return self

    def delete(self):
        """Set QLabel with delete style."""
        self.set_dayu_delete(True)
        return self

    def underline(self):
        """Set QLabel with underline style."""
        self.set_dayu_underline(True)
        return self

    @QtCore.Slot(str)
    def on_dayu_text_changed(self, value: str):
        self.setText(value)



from qtpy import QtWidgets

from qtui.dayu.divider import MDivider
from qtui.dayu.field_mixin import MFieldMixin
from qtui.dayu.switch import MSwitch
from qtui.dayu.theme import MTheme


class SwitchExample(QtWidgets.QWidget, MFieldMixin):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setWindowTitle("Examples for MSwitch")
        self._init_ui()

    def _init_ui(self):
        check_box_1 = MSwitch()
        check_box_1.setChecked(True)
        check_box_2 = MSwitch()
        check_box_3 = MSwitch()
        check_box_3.setEnabled(False)
        lay = QtWidgets.QHBoxLayout()
        lay.addWidget(check_box_1)
        lay.addWidget(check_box_2)
        lay.addWidget(check_box_3)

        size_lay = QtWidgets.QFormLayout()
        size_lay.addRow("Huge", MSwitch().huge())
        size_lay.addRow("Large", MSwitch().large())
        size_lay.addRow("Medium", MSwitch().medium())
        size_lay.addRow("Small", MSwitch().small())
        size_lay.addRow("Tiny", MSwitch().tiny())

        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addWidget(MDivider("Basic"))
        main_lay.addLayout(lay)
        main_lay.addWidget(MDivider("different size"))
        main_lay.addLayout(size_lay)
        main_lay.addStretch()
        self.setLayout(main_lay)


if __name__ == "__main__":
    from qtui.dayu.qt import application

    with application() as app:
        test = SwitchExample()
        MTheme().apply(test)
        test.show()



"""
Some helper functions for handling color and formatter.
"""

from __future__ import annotations

import datetime as dt
import math
import threading
from collections.abc import Iterable
from functools import singledispatch, wraps
from pathlib import Path
from typing import Any, TypeVar, cast

from qtpy import QtCore, QtGui, QtWidgets

from .qt import MIcon, get_scale_factor
from .types import MenuItemData

T = TypeVar("T")


def get_static_file(file_name: str) -> Path:
    """
    Args:
        file_name: The name of the file in the static folder.

    Returns:
        The full path of the file.
    """
    path = Path(__file__).parent / "static" / file_name

    if not path.is_file() or not path.exists():
        raise FileNotFoundError(f"File {path} not found")
    return path


def from_list_to_nested_dict(
    input_arg: Iterable[str], sep: str = "/"
) -> list[MenuItemData]:
    """
    A help function to convert the list of string to nested dict
    :param input_arg: a list/tuple/set of string
    :param sep: a separator to split input string
    :return: a list of nested dict
    """

    result: list[MenuItemData] = []
    for item in input_arg:
        components = item.strip(sep).split(sep)
        component_count = len(components)
        current = result
        for i, comp in enumerate(components):
            atom: MenuItemData | None = None
            for x in current:
                if x.get("value") == comp:
                    atom = x
                    break

            if atom is None:
                atom = MenuItemData(value=comp, label=comp, children=[])
                current.append(atom)
            current = atom.get("children", [])
            if i == component_count - 1:
                atom.pop("children")

    return result


def fade_color(color: str, alpha: str) -> str:
    """
    Fade color with given alpha.
    eg. fade_color('#ff0000', '10%) => 'rgba(255, 0, 0, 10%)'

    Args:
        color: string, hex digit format '#RRGGBB'
        alpha: string, percent 'number%'
    Returns:
        qss/css color format rgba(r, g, b, a)
    """
    q_color = QtGui.QColor(color)
    return f"rgba({q_color.red()}, {q_color.green()}, {q_color.blue()}, {alpha})"


def generate_color(primary_color: str | QtGui.QColor, index: int) -> str:
    """
    Reference to ant-design color system algorithm.

    Args:
        primary_color: base color. #RRGGBB
        index: color step. 1-10 from light to dark
    Returns:
        result color
    """

    hue_step = 2
    saturation_step = 16
    saturation_step2 = 5
    brightness_step1 = 5
    brightness_step2 = 15
    light_color_count = 5
    dark_color_count = 4

    def _get_hue(color: QtGui.QColor, i: int, is_light: bool) -> float:
        h_comp = color.hue()
        if 60 <= h_comp <= 240:
            hue = h_comp - hue_step * i if is_light else h_comp + hue_step * i
        else:
            hue = h_comp + hue_step * i if is_light else h_comp - hue_step * i
        if hue < 0:
            hue += 359
        elif hue >= 359:
            hue -= 359
        return hue / 359.0

    def _get_saturation(color: QtGui.QColor, i: int, is_light: bool) -> float:
        s_comp = color.saturationF() * 100
        if is_light:
            saturation = s_comp - saturation_step * i
        elif i == dark_color_count:
            saturation = s_comp + saturation_step
        else:
            saturation = s_comp + saturation_step2 * i
        saturation = min(100.0, saturation)
        if is_light and i == light_color_count and saturation > 10:
            saturation = 10
        saturation = max(6.0, saturation)
        return round(saturation * 10) / 1000.0

    def _get_value(color: QtGui.QColor, i: int, is_light: bool) -> float:
        v_comp = color.valueF()
        if is_light:
            return min((v_comp * 100 + brightness_step1 * i) / 100, 1.0)
        return max((v_comp * 100 - brightness_step2 * i) / 100, 0.0)

    light = index <= 6
    hsv_color = (
        QtGui.QColor(primary_color) if isinstance(primary_color, str) else primary_color
    )
    index = light_color_count + 1 - index if light else index - light_color_count - 1
    return QtGui.QColor.fromHsvF(
        _get_hue(hsv_color, index, light),
        _get_saturation(hsv_color, index, light),
        _get_value(hsv_color, index, light),
    ).name()


def real_model(
    item: QtCore.QAbstractItemModel
    | QtCore.QSortFilterProxyModel
    | QtCore.QModelIndex
    | None,
) -> QtCore.QAbstractItemModel | None:
    if isinstance(item, QtCore.QSortFilterProxyModel):
        return item.sourceModel()
    elif isinstance(item, QtCore.QModelIndex):
        return item.model()
    return item


def real_index(index: QtCore.QModelIndex) -> QtCore.QModelIndex:
    """
    Get the source index whenever user give a source index or proxy index.
    """
    model = index.model()
    if isinstance(model, QtCore.QSortFilterProxyModel):
        return model.mapToSource(index)
    return index


def get_obj_value(
    data_obj: dict[str, Any] | object, attr: str, default: Any = None
) -> Any:
    """Get dict's key or object's attribute with given attr"""
    if isinstance(data_obj, dict):
        return cast(dict[str, Any], data_obj).get(attr, default)
    return getattr(data_obj, attr, default)


def set_obj_value(data_obj: dict[str, Any] | object, attr: str, value: Any) -> None:
    """Set dict's key or object's attribute with given attr and value"""
    if isinstance(data_obj, dict):
        return cast(dict[str, Any], data_obj).update({attr: value})
    return setattr(data_obj, attr, value)


def has_obj_value(data_obj: dict[str, Any] | object, attr: str) -> bool:
    """Return weather dict has the given key or object has the given attribute."""
    if isinstance(data_obj, dict):
        return attr in data_obj
    return hasattr(data_obj, attr)


def apply_formatter(formatter: Any, *args: Any, **kwargs: Any) -> Any:
    """
    Used for QAbstractModel data method.
    Config a formatter for one field, apply the formatter with the index data.

    Args:
        formatter: formatter. It can be None/dict/callable or just any type of value
        args:
        kwargs:
    Returns:
        apply the formatter with args and kwargs
    """
    if formatter is None:
        return args[0]
    elif isinstance(formatter, dict):
        return cast(dict[str, Any], formatter).get(args[0], None)
    elif callable(formatter):
        return formatter(*args, **kwargs)
    return formatter


@singledispatch
def display_formatter(input_other_type: Any) -> str:
    """
    Used for QAbstractItemModel data method for Qt.DisplayRole
    Format any input value to a string.
    :param input_other_type: any type value
    :return: str
    """
    return str(input_other_type)  # this function never reached


@display_formatter.register(dict)
def _(input_dict: dict[str, Any]) -> str:
    if "name" in input_dict:
        return display_formatter(input_dict.get("name"))
    elif "code" in input_dict:
        return display_formatter(input_dict.get("code"))
    return str(input_dict)


@display_formatter.register(list)
def _(input_list: list[Any]) -> str:
    result: list[str] = []
    for i in input_list:
        result.append(str(display_formatter(i)))
    return ",".join(result)


@display_formatter.register(bytes)
def _(input_str: bytes) -> str:
    # ['utf-8', 'windows-1250', 'windows-1252', 'ISO-8859-1']
    return input_str.decode("windows-1252")
    # return obj.decode()


@display_formatter.register(str)
def _(input_unicode: str) -> str:
    return input_unicode


@display_formatter.register(type(None))
def _(input_none: None) -> str:
    return "--"


@display_formatter.register(int)
def _(input_int: int) -> str:
    return str(input_int)


@display_formatter.register(float)
def _(input_float: float) -> str:
    return f"{round(input_float, 2):.2f}"


@display_formatter.register(object)
def _(input_object: object) -> str:
    if hasattr(input_object, "name"):
        return display_formatter(input_object.name)  # pyright: ignore[reportUnknownMemberType, reportAttributeAccessIssue]
    if hasattr(input_object, "code"):
        return display_formatter(input_object.code)  # pyright: ignore[reportUnknownMemberType, reportAttributeAccessIssue]
    return str(input_object)


@display_formatter.register(dt.datetime)
def _(input_datetime: dt.datetime) -> str:
    return input_datetime.strftime("%Y-%m-%d %H:%M:%S")


def font_formatter(setting_dict: dict[str, Any]) -> QtGui.QFont:
    """
    Used for QAbstractItemModel data method for Qt.FontRole

    Args:
        setting_dict: a dict with underline and bold
    Returns:
        a QFont instance with given style
    """
    _font = QtGui.QFont()
    _font.setUnderline(setting_dict.get("underline") or False)
    _font.setBold(setting_dict.get("bold") or False)
    return _font


@singledispatch
def icon_formatter(input_other_type: Any) -> QtGui.QIcon:
    """
    Used for QAbstractItemModel data method for Qt.DecorationRole
    A helper function to easy get QIcon.
    The input can be dict/object, string, None, tuple(file_path, fill_color)

    Args:
        input_other_type:
    Returns:
        a QIcon instance
    """
    return input_other_type  # this function never reached


@icon_formatter.register(dict)
def _(input_dict: dict[str, Any]) -> QtGui.QIcon:
    attr_list = ["icon"]
    path = next((get_obj_value(input_dict, attr) for attr in attr_list), None)
    return icon_formatter(path)


@icon_formatter.register(QtGui.QIcon)
def _(input_dict: QtGui.QIcon) -> QtGui.QIcon:
    return input_dict


@icon_formatter.register(object)
def _(input_object: object) -> QtGui.QIcon:
    attr_list = ["icon"]
    path = next((get_obj_value(input_object, attr) for attr in attr_list), None)
    return icon_formatter(path)


@icon_formatter.register(str)
def _(input_string: str) -> QtGui.QIcon:
    return MIcon(input_string)


@icon_formatter.register(tuple)
def _(input_tuple: tuple[str, str]) -> QtGui.QIcon:
    return MIcon(*input_tuple)


@icon_formatter.register(type(None))
def _(input_none: None) -> QtGui.QIcon:
    return icon_formatter("confirm_fill.svg")


def overflow_format(num: int, overflow: int) -> str:
    """
    Give a integer, return a string.
    When this integer is large than given overflow, return "overflow+"
    """
    return str(num) if num <= overflow else f"{overflow}+"


def get_percent(value: int, minimum: int, maximum: int) -> float:
    """
    Get a given value's percent in the range.

    Args:
        value: value
        minimum: the range's minimum value
        maximum: the range's maximum value
    Returns:
        percent float
    """
    if minimum == maximum:
        # reference from qprogressbar.cpp
        # If max and min are equal and we get this far, it means that the
        # progress bar has one step and that we are on that step. Return
        # 100% here in order to avoid division by zero further down.
        return 100
    return max(0, min(100, (value - minimum) * 100 / (maximum - minimum)))


def get_total_page(total: int, per: int) -> int:
    """
    Get the page count.
    :param total: total count
    :param per: count per page
    :return: page count int
    """
    return math.ceil(1.0 * total / per)


def get_page_display_string(current: int, per: int, total: int) -> str:
    """
    Get the format string x - x of xx
    :param current: current page
    :param per: count per page
    :param total: total count
    :return: str
    """
    return f"{((current - 1) * per + 1) if current else 0} - {min(total, current * per)} of {total}"  # noqa: E501


def read_settings(organization: str, app_name: str) -> dict[str, Any]:
    settings = QtCore.QSettings(
        QtCore.QSettings.Format.IniFormat,
        QtCore.QSettings.Scope.UserScope,
        organization,
        app_name,
    )
    result_dict = {key: settings.value(key) for key in settings.childKeys()}
    for grp_name in settings.childGroups():
        settings.beginGroup(grp_name)
        result_dict.update(
            {grp_name + "/" + key: settings.value(key) for key in settings.childKeys()}
        )
        settings.endGroup()
    return result_dict


def get_fit_geometry() -> QtCore.QRect:
    geo = next(
        (screen.availableGeometry() for screen in QtWidgets.QApplication.screens()),
        None,
    )
    if geo is None:
        return QtCore.QRect(0, 0, 0, 0)
    return QtCore.QRect(
        geo.width() // 4, geo.height() // 4, geo.width() // 2, geo.height() // 2
    )


def convert_to_round_pixmap(orig_pix: QtGui.QPixmap) -> QtGui.QPixmap:
    w = min(orig_pix.width(), orig_pix.height())
    pix_map = QtGui.QPixmap(w, w)
    pix_map.fill(QtCore.Qt.GlobalColor.transparent)

    painter = QtGui.QPainter(pix_map)
    painter.setRenderHints(
        QtGui.QPainter.RenderHint.Antialiasing
        | QtGui.QPainter.RenderHint.SmoothPixmapTransform
    )

    path = QtGui.QPainterPath()
    path.addEllipse(0, 0, w, w)
    painter.setClipPath(path)
    painter.drawPixmap(0, 0, w, w, orig_pix)
    return pix_map


def generate_text_pixmap(
    width: int,
    height: int,
    text: str,
    alignment: QtCore.Qt.AlignmentFlag = QtCore.Qt.AlignmentFlag.AlignCenter,
    bg_color: str = "",
) -> QtGui.QPixmap:
    from .theme import MTheme

    bg_color = bg_color or MTheme().background_in_color
    # draw a pixmap with text
    pix_map = QtGui.QPixmap(width, height)
    pix_map.fill(QtGui.QColor(bg_color))
    painter = QtGui.QPainter(pix_map)
    painter.setRenderHints(QtGui.QPainter.RenderHint.TextAntialiasing)
    font = painter.font()
    font.setFamily(MTheme().font_family)
    painter.setFont(font)
    painter.setPen(QtGui.QPen(QtGui.QColor(MTheme().secondary_text_color)))

    font_metrics = painter.fontMetrics()
    text_width = font_metrics.horizontalAdvance(text)
    text_height = font_metrics.height()
    x = width // 2 - text_width // 2
    y = height // 2 - text_height // 2
    if alignment & QtCore.Qt.AlignmentFlag.AlignLeft:
        x = 0
    elif alignment & QtCore.Qt.AlignmentFlag.AlignRight:
        x = width - text_width
    elif alignment & QtCore.Qt.AlignmentFlag.AlignTop:
        y = 0
    elif alignment & QtCore.Qt.AlignmentFlag.AlignBottom:
        y = height - text_height

    painter.drawText(x, y, text)
    painter.end()
    return pix_map


def get_color_icon(color: str | QtGui.QColor, size: int = 24) -> QtGui.QIcon:
    scale_x, _ = get_scale_factor()
    pix = QtGui.QPixmap(int(size * scale_x), int(size * scale_x))
    q_color = color
    if isinstance(color, str):
        if color.startswith("#"):
            q_color = QtGui.QColor(str)
        elif color.count(",") == 2:
            q_color = QtGui.QColor(*tuple(map(int, color.split(","))))
    pix.fill(q_color)
    return QtGui.QIcon(pix)


def str_to_qbytearray(string: str):
    return QtCore.QByteArray(string.encode())


def qbytearray_to_str(qbytearray: QtCore.QByteArray):
    return str(qbytearray.data().decode())  # pyright: ignore[reportUnknownMemberType, reportUnknownArgumentType, reportAttributeAccessIssue]


def singleton(cls: type[T]) -> type[T]:
    """스레드 안전한 싱글톤 데코레이터"""
    instances: dict[type[T], T] = {}
    lock = threading.Lock()

    @wraps(cls)
    def get_instance(*args: Any, **kwargs: Any) -> T:
        if cls not in instances:
            with lock:
                if cls not in instances:
                    instances[cls] = cls(*args, **kwargs)
        return instances[cls]

    # 클래스 메서드로 명시적 접근 제공
    cls.get_instance = classmethod(lambda c, *a, **k: get_instance(*a, **k))  # pyright: ignore[reportAttributeAccessIssue]
    cls.clear_instance = classmethod(lambda c: instances.pop(cls, None))  # pyright: ignore[reportAttributeAccessIssue]

    return get_instance  # pyright: ignore[reportReturnType]



from qtpy import QtCore, QtWidgets

from .mixin import cursor_mixin, stacked_animation_mixin


@cursor_mixin
class MTabBar(QtWidgets.QTabBar):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent=parent)
        self.setDrawBase(False)

    def tabSizeHint(self, index: int) -> QtCore.QSize:  # noqa: N802
        tab_text = self.tabText(index)
        if self.tabsClosable():
            return QtCore.QSize(
                self.fontMetrics().boundingRect(tab_text).width() + 70,
                self.fontMetrics().height() + 20,
            )
        else:
            return QtCore.QSize(
                self.fontMetrics().boundingRect(tab_text).width() + 50,
                self.fontMetrics().height() + 20,
            )


@stacked_animation_mixin
class MTabWidget(QtWidgets.QTabWidget):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent=parent)
        self.bar = MTabBar()
        self.setTabBar(self.bar)

    def disable_animation(self) -> None:
        self.currentChanged.disconnect(self._play_anim)  # pyright: ignore[reportUnknownMemberType, reportUnknownArgumentType, reportAttributeAccessIssue]



from qtpy import QtWidgets

import examples.dayu._mock_data as mock
from qtui.dayu.divider import MDivider
from qtui.dayu.field_mixin import MFieldMixin
from qtui.dayu.item_model import MSortFilterModel, MTableModel
from qtui.dayu.item_view import MListView
from qtui.dayu.line_edit import MLineEdit
from qtui.dayu.theme import MTheme


class ListViewExample(QtWidgets.QWidget, MFieldMixin):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self._init_ui()

    def _init_ui(self):
        table_small = MListView(size=MTheme().small)
        table_default = MListView()
        table_large = MListView(size=MTheme().large)

        model_1 = MTableModel()
        model_1.set_header_list(mock.header_list)
        model_sort = MSortFilterModel()
        model_sort.setSourceModel(model_1)
        table_small.setModel(model_sort)
        table_default.setModel(model_sort)
        table_large.setModel(model_sort)
        model_sort.set_header_list(mock.header_list)
        table_small.set_header_list(mock.header_list)
        table_default.set_header_list(mock.header_list)
        table_large.set_header_list(mock.header_list)
        model_1.set_data_list(mock.data_list)

        line_edit = MLineEdit().search_with_close().small()
        line_edit.textChanged.connect(model_sort.set_search_pattern)

        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addWidget(line_edit)
        main_lay.addWidget(MDivider("Small Size"))
        main_lay.addWidget(table_small)
        main_lay.addWidget(MDivider("Default Size"))
        main_lay.addWidget(table_default)
        main_lay.addWidget(MDivider("Large Size"))
        main_lay.addWidget(table_large)
        main_lay.addStretch()
        self.setLayout(main_lay)


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = ListViewExample()
        MTheme().apply(test)
        test.show()



from __future__ import annotations

import re
from collections.abc import Callable
from dataclasses import dataclass
from typing import Any, NamedTuple, NotRequired, TypedDict

from qtpy import QtCore, QtGui, QtWidgets


class MBreadcrumbData(TypedDict):
    text: NotRequired[str]
    svg: NotRequired[str]
    tooltip: NotRequired[str]
    clicked: NotRequired[Callable[[], None]]


class MButtonGroupData(TypedDict):
    # default properties
    dayu_size: NotRequired[int]
    dayu_type: NotRequired[str]
    text: NotRequired[str]
    icon: NotRequired[QtGui.QIcon]
    data: NotRequired[Any]
    checked: NotRequired[bool]
    shortcut: NotRequired[str]
    tooltip: NotRequired[str]
    checkable: NotRequired[bool]
    clicked: NotRequired[Callable[..., Any]]
    toggled: NotRequired[Callable[..., Any]]

    # mtoolbutton properties
    svg: NotRequired[str]


class MCollapseData(TypedDict):
    title: NotRequired[str]
    content: NotRequired[QtWidgets.QWidget | None]
    expand: NotRequired[bool]
    closable: NotRequired[bool]
    parent: NotRequired[QtWidgets.QWidget | None]


class MCardData(TypedDict):
    title: NotRequired[str]
    image: NotRequired[QtGui.QPixmap | None]
    size: NotRequired[int]
    extra: NotRequired[bool]
    extra_menus: NotRequired[list[tuple[str, Callable[[], None]]] | None]
    border: NotRequired[bool]
    parent: NotRequired[QtWidgets.QWidget | None]


class MMetaCardData(TypedDict):
    title: NotRequired[str]
    description: NotRequired[str]
    avatar: NotRequired[QtGui.QPixmap | None]
    cover: NotRequired[QtGui.QPixmap | None]
    extra: NotRequired[bool]
    extra_menus: NotRequired[list[tuple[str, Callable[[], None]]] | None]
    border: NotRequired[bool]
    parent: NotRequired[QtWidgets.QWidget | None]


class MCardTitleProps(NamedTuple):
    level: int
    padding: int


class MixinData(TypedDict):
    data_name: str
    widget: QtWidgets.QWidget
    widget_property: str
    index: int | None
    callback: Callable[[], Any] | None


class MixinProps(TypedDict):
    value: Callable[[], Any] | Any
    required: bool
    bind: list[MixinData]


class MixinComputed(TypedDict):
    value: Any
    getter: Callable[[], Any] | None
    setter: Callable[[Any], None] | None
    required: bool
    bind: list[MixinData]


ModelData = dict[str, Any]


class HeaderData(TypedDict):
    label: str
    key: str
    width: NotRequired[float]
    default_filter: NotRequired[bool]
    editable: NotRequired[bool]
    selectable: NotRequired[bool]
    searchable: NotRequired[bool]
    checkable: NotRequired[bool]
    exclusive: NotRequired[bool]
    order: NotRequired[str]  # asc, desc
    color: NotRequired[
        Callable[[Any, ModelData], QtGui.QColor | str] | str
    ]  # Any is the data_obj
    bg_color: NotRequired[Callable[[Any, ModelData], QtGui.QColor | str] | str]
    display: NotRequired[Callable[[Any, ModelData], str]]
    align: NotRequired[str]  # left, center, right
    font: NotRequired[
        Callable[[Any, ModelData], dict[str, Any]]
    ]  # {underline: bool, bold: bool}
    icon: NotRequired[Callable[[Any, ModelData], Any] | str]
    tooltip: NotRequired[Callable[[Any, ModelData], str]]
    size: NotRequired[Callable[[Any, ModelData], QtCore.QSize]]
    hide: NotRequired[bool]
    reg: NotRequired[re.Pattern[str] | None]


class MenuItemData(TypedDict):
    value: str | int | float | MenuItemData
    label: str
    children: NotRequired[list[MenuItemData]]


@dataclass
class ItemViewMenuEvent:
    view: QtWidgets.QAbstractItemView
    selection: list[QtCore.QItemSelection]
    extra: Any



"""
Example code for MDateTimeEdit, MDoubleSpinBox, MSpinBox, MTimeEdit, MDateEdit
"""

from qtpy import QtWidgets

from qtui.dayu.divider import MDivider
from qtui.dayu.spin_box import (
    MDateEdit,
    MDateTimeEdit,
    MDoubleSpinBox,
    MSpinBox,
    MTimeEdit,
)


class SpinBoxExample(QtWidgets.QWidget):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setWindowTitle("Examples for Spin Box")

        main_lay = QtWidgets.QVBoxLayout()
        class_list = [MSpinBox, MDoubleSpinBox, MDateTimeEdit, MDateEdit, MTimeEdit]
        for cls in class_list:
            main_lay.addWidget(MDivider(cls.__name__))
            lay = QtWidgets.QHBoxLayout()
            lay.addWidget(cls().large())
            lay.addWidget(cls().medium())
            lay.addWidget(cls().small())
            main_lay.addLayout(lay)

        main_lay.addWidget(MDivider("Pop Calendar Widget"))
        date_time_edit = MDateTimeEdit()
        date_time_edit.setCalendarPopup(True)
        date_edit = MDateEdit()
        date_edit.setCalendarPopup(True)
        time_edit = MTimeEdit()
        time_edit.setCalendarPopup(True)
        date_lay = QtWidgets.QHBoxLayout()
        date_lay.addWidget(date_time_edit)
        date_lay.addWidget(date_edit)
        date_lay.addWidget(time_edit)
        main_lay.addLayout(date_lay)

        main_lay.addStretch()
        self.setLayout(main_lay)


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = SpinBoxExample()
        MTheme().apply(test)
        test.show()



from qtpy import QtWidgets

from qtui.dayu.divider import MDivider
from qtui.dayu.field_mixin import MFieldMixin
from qtui.dayu.label import MLabel


class DividerExample(QtWidgets.QWidget, MFieldMixin):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setWindowTitle("Examples for MDivider")
        self._init_ui()

    def _init_ui(self):
        div1 = MDivider()
        div2 = MDivider("With Text")
        div3 = MDivider.left("Left Text")
        div4 = MDivider.center("Center Text")
        div5 = MDivider.right("Right Text")
        div6 = MDivider.vertical()
        div7 = MDivider.vertical()
        div8 = MDivider.left("orientation=Qt.Vertical")
        label1 = MLabel("Maya").strong()
        label2 = MLabel("Nuke").underline()
        label3 = MLabel("Houdini").mark()
        sub_lay = QtWidgets.QHBoxLayout()
        sub_lay.addWidget(label1)
        sub_lay.addWidget(div6)
        sub_lay.addWidget(label2)
        sub_lay.addWidget(div7)
        sub_lay.addWidget(label3)
        sub_lay.addStretch()

        some_text = (
            "Steven Paul Jobs was an American entrepreneur and business magnate."
        )
        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addWidget(MLabel(some_text))
        main_lay.addWidget(div1)
        main_lay.addWidget(MLabel(some_text))
        main_lay.addWidget(div2)
        main_lay.addWidget(MLabel(some_text))
        main_lay.addWidget(div3)
        main_lay.addWidget(MLabel(some_text))
        main_lay.addWidget(div4)
        main_lay.addWidget(MLabel(some_text))
        main_lay.addWidget(div5)
        main_lay.addLayout(sub_lay)
        main_lay.addWidget(div8)
        main_lay.addStretch()
        self.setLayout(main_lay)

    def computed_text(self):
        return "Clicked: " + str(self.field("count"))

    def slot_change_divider_text(self):
        self.set_field("count", self.field("count") + 1)


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = DividerExample()
        MTheme().apply(test)
        test.show()
